Package: shinyglass
Type: Package
Title: Liquid Glass Design Themes for 'shiny' Applications
Version: 0.2.0
Authors@R: c(
    person("Eric", "Anderson",
           email = "eric.ray.anderson@gmail.com",
           role = c("aut", "cre", "cph"))
    )
Description: Provides drop-in Liquid Glass themes for 'shiny'. Call
    glass_theme() and pass the result as theme = to fluidPage(),
    navbarPage(), or any 'bslib'-aware page function to get
    translucent surfaces, backdrop blur, and system typography on
    'Bootstrap' components. Includes light and dark presets with
    runtime switching and an OS-following 'auto' mode, an iOS-style
    intensity control from Ultra Clear to Tinted
    (glass_intensity_slider()), and options for accent color, blur,
    corner radius, and motion or tint behavior.
License: GPL-3
URL: https://ericrayanderson.github.io/shinyglass/,
        https://github.com/ericrayanderson/shinyglass
BugReports: https://github.com/ericrayanderson/shinyglass/issues
Encoding: UTF-8
Language: en-US
VignetteBuilder: knitr
Imports: bslib (>= 0.5.0), htmltools (>= 0.5.0), sass (>= 0.4.0), shiny
        (>= 1.5.0)
Suggests: DT, ggplot2, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/Needs/website: pkgdown
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-20 15:38:39 UTC; box
Author: Eric Anderson [aut, cre, cph]
Maintainer: Eric Anderson <eric.ray.anderson@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 05:40:42 UTC

# Generated by roxygen2: do not edit by hand

export(glass_intensity_slider)
export(glass_preset_input)
export(glass_resolved_preset)
export(glass_theme)
export(glass_theme_toggle)
export(observe_glass_intensity)
export(observe_glass_preset_input)
export(observe_glass_theme_toggle)
export(update_glass_theme)

# shinyglass

<!-- badges: start -->
[![License: GPL-3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.r-project.org/Licenses/GPL-3)
[![R-CMD-check](https://github.com/ericrayanderson/shinyglass/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ericrayanderson/shinyglass/actions/workflows/R-CMD-check.yaml)
[![visual-qa](https://github.com/ericrayanderson/shinyglass/actions/workflows/visual-qa.yaml/badge.svg)](https://github.com/ericrayanderson/shinyglass/actions/workflows/visual-qa.yaml)
<!-- badges: end -->

[Liquid Glass](https://developer.apple.com/documentation/technologyoverviews/liquid-glass) themes for [Shiny](https://shiny.posit.co/). `glass_theme()` returns a [bslib](https://rstudio.github.io/bslib/) theme with translucent surfaces, backdrop blur, and system typography. Pass it as `theme = glass_theme()` to `fluidPage()`, `navbarPage()`, or other page functions that accept a bslib theme.

**Runtime controls (no page reload):**

- **Light / Dark / Auto** — `glass_theme_toggle()` + `update_glass_theme(preset = …)`, or `preset = "auto"` to follow the OS
- **Liquid Glass intensity** — iOS 27-style **Ultra Clear → Tinted** via `glass_intensity_slider()` (and `glass_theme(intensity = 0–1)`)

[Documentation](https://ericrayanderson.github.io/shinyglass/) · [GitHub](https://github.com/ericrayanderson/shinyglass) · **[Live demos](#live-demos)**

### Liquid Glass intensity

Drop in `glass_intensity_slider()` for the same continuous control as iOS 27 **Settings → Appearance → Liquid Glass**. Surfaces update live as you drag from **Ultra Clear** (`0`) to **Tinted** (`1`) — no page reload.

<p align="center">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider.gif" width="72%" alt="Liquid Glass intensity slider animating Ultra Clear to Tinted">
</p>

<p align="center">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider.png" width="48%" alt="Liquid Glass intensity slider, light">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider-dark.png" width="48%" alt="Liquid Glass intensity slider, dark">
</p>

<p align="center">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider-clear.png" width="32%" alt="Ultra Clear intensity">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider-tinted.png" width="32%" alt="Tinted intensity">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/intensity-slider-dark-tinted.png" width="32%" alt="Tinted intensity, dark">
</p>

<p align="center">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/bslib-dashboard.png" width="48%" alt="Glass dashboard, light">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/bslib-dashboard-dark.png" width="48%" alt="Glass dashboard, dark">
</p>

<p align="center">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/plotly-gt.png" width="48%" alt="plotly + gt, light">
<img src="https://raw.githubusercontent.com/ericrayanderson/shinyglass/main/man/figures/plotly-gt-dark.png" width="48%" alt="plotly + gt, dark">
</p>

## Live demos

Public apps on [shinyapps.io](https://www.shinyapps.io/) (free tier may take a few seconds to wake):

| App | What it shows |
|-----|----------------|
| [Demo + theme toggle](https://ericrayanderson.shinyapps.io/shinyglass-demo/) | Core glass UI + **`glass_intensity_slider()`** + Light / Dark / Auto |
| [bslib dashboard](https://ericrayanderson.shinyapps.io/shinyglass-dashboard/) | `page_sidebar`, value boxes, plots, DT + intensity in the sidebar |
| [Inputs gallery](https://ericrayanderson.shinyapps.io/shinyglass-inputs/) | Built-in Shiny inputs under glass |
| [plotly + gt](https://ericrayanderson.shinyapps.io/shinyglass-plotly-gt/) | plotly modebar + gt tables under glass |
| [Olympic medals](https://ericrayanderson.shinyapps.io/shinyglass-olympics/) | denser dreamRs-style dashboard (glass port) |

## Install

```r
# once on CRAN:
install.packages("shinyglass")

# development version:
# remotes::install_github("ericrayanderson/shinyglass")
```

## Quick start

```r
library(shiny)
library(shinyglass)

ui <- fluidPage(
  theme = glass_theme(preset = "auto", intensity = 0.45),
  titlePanel("Liquid Glass"),
  # iOS 27-style Ultra Clear → Tinted (updates live on the client)
  glass_intensity_slider("glass_intensity"),
  # Light / Dark / Auto buttons
  glass_theme_toggle(selected = "auto"),
  sliderInput("n", "Bars", 5, 30, 15),
  plotOutput("plot")
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)
  # optional: keep server session intensity in sync (client already updates live)
  observe_glass_intensity(input, session, "glass_intensity")
  output$plot <- renderPlot(barplot(seq_len(input$n), col = "#007AFF", border = NA))
}

shinyApp(ui, server)
```

Set the starting material with `glass_theme(intensity = …)` (`0` Ultra Clear … `1` Tinted), or drive it from the server with `update_glass_theme(session, intensity = 0.8)`.

See the [theming article](https://ericrayanderson.github.io/shinyglass/articles/theming.html) for knobs (`intensity`, `tint`, `specular`, `nav_morph`) and CSS variables.

```r
# re-deploy live demos (requires rsconnect; installs shinyglass from GitHub):
# remotes::install_github("ericrayanderson/shinyglass")
# Rscript inst/scripts/deploy-shinyapps-demos.R
# Rscript inst/scripts/deploy-shinyapps-demos.R --apps=demo,dashboard,intensity
#
# visual QA (contrast + DT/selectize structure, light+dark matrix):
# Rscript inst/scripts/audit-glass-contrast.R
# see inst/scripts/VISUAL-QA.md
```

## Local examples

```r
shiny::runApp(system.file("examples", "demo-app.R", package = "shinyglass"))
shiny::runApp(system.file("examples", "bslib-dashboard.R", package = "shinyglass"))
shiny::runApp(system.file("examples", "intensity-slider-demo.R", package = "shinyglass"))
shiny::runApp(system.file("examples", "inputs-gallery.R", package = "shinyglass"))
shiny::runApp(system.file("examples", "chrome-kitchen-sink.R", package = "shinyglass"))
# plotly + gt (+ waiter if installed):
# shiny::runApp(system.file("examples", "plotly-gt-demo.R", package = "shinyglass"))
```

## Shiny for Python (experimental)

A parallel package lives under
[`python/`](https://github.com/ericrayanderson/shinyglass/tree/main/python)
(not on CRAN/PyPI yet). See that README for wheel builds.

# iOS 27 Liquid Glass intensity slider demo.
# Ultra Clear (0) ←————→ Tinted (1)
#
# Run:
#   shiny::runApp(system.file("examples", "intensity-slider-demo.R", package = "shinyglass"))
#
# Dark:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "intensity-slider-demo.R", package = "shinyglass"))

library(shiny)
library(bslib)
library(shinyglass)

if (!requireNamespace("ggplot2", quietly = TRUE)) {
  stop("ggplot2 is required. install.packages('ggplot2')")
}
library(ggplot2)

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "light"),
  c("light", "dark", "auto")
)

ui <- page_fillable(
  theme = glass_theme(
    preset = glass_preset,
    primary = "#007AFF",
    intensity = 0.45
  ),
  padding = "1rem",
  gap = "0.85rem",
  layout_columns(
    col_widths = breakpoints(xs = c(12, 12), md = c(4, 8)),
    card(
      card_header("Liquid Glass intensity"),
      tags$p(
        class = "text-muted",
        style = "font-size:0.9rem;margin-bottom:0.75rem;",
        "iOS 27 Appearance → Liquid Glass. Drag Ultra Clear → Tinted; ",
        "surfaces update live (no reload)."
      ),
      glass_intensity_slider(
        "glass_intensity",
        label = NULL,
        value = 0.45,
        preview = TRUE
      ),
      tags$hr(),
      glass_theme_toggle(selected = glass_preset),
      tags$hr(),
      selectInput(
        "accent",
        "Accent",
        choices = c(
          "Blue" = "#007AFF",
          "Purple" = "#AF52DE",
          "Orange" = "#FF9500",
          "Green" = "#34C759"
        ),
        selected = "#007AFF",
        width = "100%"
      ),
      sliderInput("n", "Points", 40, 200, 90, width = "100%"),
      checkboxInput("show_smooth", "Density curve", TRUE),
      tags$p(
        class = "text-muted",
        style = "font-size:0.78rem;margin-top:0.75rem;margin-bottom:0;",
        textOutput("intensity_label", inline = TRUE)
      )
    ),
    div(
      layout_column_wrap(
        width = "9rem",
        fill = FALSE,
        gap = "0.65rem",
        value_box(
          title = "Look",
          value = textOutput("vb_look"),
          theme = "primary"
        ),
        value_box(
          title = "Intensity",
          value = textOutput("vb_intensity"),
          theme = "info"
        ),
        value_box(
          title = "Points",
          value = textOutput("vb_n"),
          theme = "success"
        )
      ),
      tags$div(style = "height:0.75rem;"),
      card(
        full_screen = TRUE,
        card_header("Content under glass"),
        plotOutput("hero", height = "300px")
      ),
      tags$div(style = "height:0.75rem;"),
      layout_column_wrap(
        width = 1 / 2,
        fill = FALSE,
        gap = "0.65rem",
        card(
          card_header("Chrome"),
          tags$p(
            style = "margin:0;font-size:0.9rem;",
            "Floating chrome uses true glass — watch the fill thicken as you tint."
          )
        ),
        card(
          card_header("Content"),
          tags$p(
            style = "margin:0;font-size:0.9rem;",
            "Content cards stay slightly denser so hierarchy stays clear."
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)
  observe_glass_intensity(input, session, "glass_intensity")

  observeEvent(input$accent, {
    update_glass_theme(session, primary = input$accent)
  }, ignoreInit = TRUE)

  intensity_val <- reactive({
    v <- input$glass_intensity
    if (is.null(v) || length(v) != 1L || is.na(v)) 0.45 else as.numeric(v)
  })

  output$intensity_label <- renderText({
    v <- intensity_val()
    sprintf(
      "Current intensity: %.0f%%  ·  %s",
      100 * v,
      if (v < 0.33) "Ultra Clear" else if (v > 0.66) "Tinted" else "Balanced"
    )
  })

  output$vb_look <- renderText({
    v <- intensity_val()
    if (v < 0.33) "Ultra Clear" else if (v > 0.66) "Tinted" else "Balanced"
  })

  output$vb_intensity <- renderText({
    sprintf("%.0f%%", 100 * intensity_val())
  })

  output$vb_n <- renderText({
    as.character(input$n)
  })

  output$hero <- renderPlot({
    set.seed(1)
    n <- input$n
    df <- data.frame(
      x = rnorm(n, 5, 1.2),
      g = sample(c("A", "B", "C"), n, replace = TRUE)
    )
    cols <- c(A = "#007AFF", B = "#AF52DE", C = "#FF9500")
    p <- ggplot(df, aes(x, fill = g, color = g)) +
      geom_histogram(bins = 22, alpha = 0.72, position = "identity", linewidth = 0) +
      scale_fill_manual(values = cols) +
      scale_color_manual(values = cols) +
      labs(x = NULL, y = NULL, fill = NULL, color = NULL) +
      theme_void(base_size = 14) +
      theme(
        legend.position = "bottom",
        plot.background = element_rect(fill = "transparent", color = NA),
        panel.background = element_rect(fill = "transparent", color = NA)
      )
    if (isTRUE(input$show_smooth)) {
      p <- p + geom_density(aes(y = after_stat(count)), linewidth = 1.1, fill = NA)
    }
    p
  }, bg = "transparent", res = 96)
}

shinyApp(ui, server)

# Liquid Glass reference app for shinyglass visual validation.
# Exercises: page_sidebar overlay, pill tabs, content-aware tinting, DT tables.
#
# Run:
#   shiny::runApp(system.file("examples", "apple-glass-reference.R", package = "shinyglass"))
#
# Dark preset:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "apple-glass-reference.R", package = "shinyglass"))

library(shiny)
library(bslib)
library(shinyglass)

if (!requireNamespace("ggplot2", quietly = TRUE)) {
  stop("ggplot2 is required for this reference app.")
}
if (!requireNamespace("DT", quietly = TRUE)) {
  stop("DT is required for this reference app.")
}
library(ggplot2)

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "light"),
  c("light", "dark")
)

palette_colors <- list(
  Ocean = c("#007AFF", "#5AC8FA", "#34C759"),
  Sunset = c("#FF9500", "#FF2D55", "#AF52DE"),
  Forest = c("#34C759", "#30B0C7", "#5856D6")
)

ui <- page_sidebar(
  title = "Liquid Glass Reference",
  theme = glass_theme(preset = glass_preset),
  fillable = TRUE,
  sidebar = sidebar(
    title = "Controls",
    width = 280,
    selectInput(
      "palette",
      "Chart palette",
      choices = names(palette_colors),
      selected = "Ocean"
    ),
    selectizeInput(
      "species",
      "Species (selectize)",
      choices = c("setosa", "versicolor", "virginica"),
      selected = "setosa"
    ),
    sliderInput("bins", "Histogram bins", min = 8, max = 40, value = 18),
    checkboxInput("density", "Show density curve", TRUE),
    glass_preset_input(
      "preset",
      label = "Theme preset",
      selected = glass_preset,
      choices = c("Light" = "light", "Dark" = "dark"),
      width = "100%"
    ),
    actionButton("notify", "Show notification", class = "btn-primary"),
    tags$hr(),
    tags$small(
      class = "text-muted",
      "Theme preset switches glass live. ",
      "Launch with SHINYGLASS_PRESET=dark to start in dark."
    )
  ),
  p(
    class = "text-muted",
    "Validates shinyglass Liquid Glass goals: translucent chrome ",
    "floating above full-bleed content, informed by surrounding color."
  ),
  navset_card_tab(
    id = "tabs",
    nav_panel(
      "Chart",
      div(
        class = "glass-content-hero",
        plotOutput("hero_plot", height = "340px")
      )
    ),
    nav_panel(
      "Data",
      DT::DTOutput("hero_dt")
    ),
    nav_panel(
      "Checklist",
      tags$ul(
        tags$li("page_sidebar overlay — glass sidebar floats above full-bleed main"),
        tags$li("Pill tab bar — compact on scroll down, expand on scroll up"),
        tags$li("Content-aware tint sampled from the chart"),
        tags$li("Selectize dropdown with glass menu styling"),
        tags$li("DT table with glass wrapper, striped rows, and glass pagination"),
        tags$li("Pointer-driven specular highlights on glass surfaces"),
        tags$li("Dark preset via glass_theme(preset = \"dark\")"),
        tags$li("Compare against ", tags$a(
          href = "https://developer.apple.com/documentation/TechnologyOverviews/adopting-liquid-glass",
          "Adopting Liquid Glass design notes"
        ))
      ),
      tags$p(tags$strong("Scroll this tab"), " to test navigation morph."),
      tags$div(
        style = "min-height: 120vh; padding-bottom: 4rem;",
        lapply(seq_len(18), function(i) {
          tags$p(paste("Scroll section", i, "— navigation should compact while scrolling down."))
        })
      )
    )
  )
)

server <- function(input, output, session) {
  plot_data <- reactive({
    cols <- palette_colors[[input$palette]]
    iris |>
      subset(Species == input$species) |>
      transform(accent = cols[1], accent2 = cols[2])
  })

  observe_glass_preset_input(input, session, "preset")

  output$hero_plot <- renderPlot({
    df <- plot_data()
    cols <- palette_colors[[input$palette]]
    text_col <- if (identical(glass_resolved_preset(input), "dark")) "#f5f5f7" else "#1d1d1f"

    p <- ggplot(df, aes(x = Sepal.Length, y = Sepal.Width, color = Species)) +
      geom_point(size = 3, alpha = 0.85) +
      scale_color_manual(values = cols) +
      labs(
        title = paste("Iris —", input$palette, "palette"),
        x = "Sepal length",
        y = "Sepal width"
      ) +
      theme_minimal(base_size = 14) +
      theme(
        panel.background = element_rect(fill = "transparent", color = NA),
        plot.background = element_rect(fill = "transparent", color = NA),
        legend.background = element_rect(fill = "transparent", color = NA),
        legend.position = "bottom",
        text = element_text(color = text_col),
        axis.text = element_text(color = text_col),
        axis.title = element_text(color = text_col),
        plot.title = element_text(color = text_col)
      )

    if (isTRUE(input$density)) {
      p <- p + geom_density(
        aes(x = Sepal.Length),
        inherit.aes = FALSE,
        fill = cols[[2]],
        alpha = 0.25,
        color = NA
      )
    }

    print(p)
  }, bg = "transparent", height = 340)

  output$hero_dt <- DT::renderDT({
    DT::datatable(
      plot_data()[, c("Sepal.Length", "Sepal.Width", "Petal.Length", "Species")],
      options = list(
        pageLength = 8,
        dom = "lftip",
        lengthMenu = c(5, 8, 12, 20)
      ),
      rownames = FALSE,
      class = "stripe hover cell-border"
    )
  })

  observeEvent(input$notify, {
    showNotification(
      paste("Glass tint driven by", input$palette, "content"),
      type = "message",
      duration = 4
    )
  })
}

shinyApp(ui, server)

# Overlay / chrome kitchen sink for shinyglass.
#
# Exercises surfaces that only exist after a click, plus page functions the
# package advertises but no other example uses: page_navbar, nav_menu,
# accordion, tooltip/popover, input_switch, datepicker popup, server modal,
# notification types, file Browse, native multi-select, inline checks.
#
# Run:
#   shiny::runApp(system.file("examples", "chrome-kitchen-sink.R", package = "shinyglass"))
#
# Dark preset:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "chrome-kitchen-sink.R", package = "shinyglass"))

library(shiny)
library(bslib)
library(shinyglass)

`%||%` <- function(x, y) {
  if (is.null(x) || length(x) == 0L || (is.character(x) && !nzchar(x[[1]]))) y else x
}

bslib_has <- function(fn) {
  exists(fn, envir = asNamespace("bslib"), inherits = FALSE)
}

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "auto"),
  c("light", "dark", "auto")
)

maybe <- function(ok, expr) {
  if (isTRUE(ok)) expr else NULL
}

tooltip_ui <- maybe(bslib_has("tooltip"), {
  tooltip(
    actionButton("tip_btn", "Hover tooltip", class = "btn-outline-primary"),
    "Tooltip on glass"
  )
})

popover_ui <- maybe(bslib_has("popover"), {
  popover(
    actionButton("pop_btn", "Open popover", class = "btn-outline-primary"),
    title = "Popover",
    "Popover body on the glass menu surface."
  )
})

switch_ui <- maybe(bslib_has("input_switch"), {
  input_switch("sw", "input_switch", value = TRUE)
})

accordion_ui <- maybe(bslib_has("accordion") && bslib_has("accordion_panel"), {
  accordion(
    id = "acc",
    open = "One",
    accordion_panel("One", "First accordion panel — glass header + body."),
    accordion_panel("Two", "Second accordion panel opened by the contrast audit.")
  )
})

navsets_ui <- tagList(
  if (bslib_has("navset_pill")) {
    navset_pill(
      nav_panel("Pill A", p("navset_pill panel A")),
      nav_panel("Pill B", p("navset_pill panel B"))
    )
  },
  tags$hr(),
  if (bslib_has("navset_underline")) {
    navset_underline(
      nav_panel("Under A", p("navset_underline panel A")),
      nav_panel("Under B", p("navset_underline panel B"))
    )
  }
)

ui <- page_navbar(
  title = "Glass chrome",
  id = "nav",
  theme = glass_theme(preset = glass_preset),
  fillable = FALSE,
  nav_panel(
    "Overlays",
    layout_column_wrap(
      width = "16rem",
      heights_equal = "row",
      gap = "0.75rem",
      card(
        card_header("Dates"),
        dateInput("date", "dateInput", value = Sys.Date(), width = "100%"),
        dateRangeInput(
          "date_range",
          "dateRangeInput",
          start = Sys.Date() - 7,
          end = Sys.Date(),
          width = "100%"
        )
      ),
      card(
        card_header("File"),
        fileInput("file", "fileInput", accept = c(".txt", ".csv"), width = "100%")
      ),
      card(
        card_header("Notifications"),
        div(
          class = "d-flex flex-wrap gap-2",
          actionButton("notify_default", "default"),
          actionButton("notify_message", "message", class = "btn-primary"),
          actionButton("notify_warning", "warning", class = "btn-warning"),
          actionButton("notify_error", "error", class = "btn-danger")
        )
      ),
      card(
        card_header("Server modal"),
        actionButton("show_server_modal", "showModal()", class = "btn-outline-primary"),
        div(
          class = "mt-2 d-flex flex-wrap gap-2 align-items-center",
          actionButton("disabled_btn", "Disabled", class = "btn-secondary", disabled = TRUE),
          tags$button(
            type = "button",
            class = "btn-close",
            id = "lone_close",
            `aria-label` = "Close"
          )
        )
      )
    )
  ),
  nav_panel(
    "bslib",
    layout_column_wrap(
      width = "16rem",
      heights_equal = "row",
      gap = "0.75rem",
      card(
        card_header("Accordion"),
        accordion_ui %||% p(class = "text-muted", "bslib::accordion() not available.")
      ),
      card(
        card_header("Tooltip / popover / switch"),
        div(class = "d-flex flex-wrap gap-2 mb-2", tooltip_ui, popover_ui),
        switch_ui %||% p(class = "text-muted", "input_switch() not available.")
      ),
      card(
        card_header("Navsets"),
        navsets_ui
      )
    )
  ),
  nav_menu(
    title = "More",
    nav_panel(
      "Selects",
      layout_column_wrap(
        width = "16rem",
        card(
          card_header("Native multi + inline"),
          selectInput(
            "multi",
            "selectInput(multiple, selectize = FALSE)",
            letters[1:8],
            selected = c("a", "c"),
            multiple = TRUE,
            selectize = FALSE,
            width = "100%"
          ),
          checkboxGroupInput(
            "inline_cb",
            "inline checkboxGroup",
            c("Email", "SMS", "Push"),
            selected = "Email",
            inline = TRUE
          ),
          radioButtons(
            "inline_radio",
            "inline radio",
            c("S", "M", "L"),
            selected = "M",
            inline = TRUE
          )
        ),
        card(
          card_header("Classic well"),
          wellPanel(
            p("wellPanel under glass_theme()."),
            sliderInput("well_n", "n", 1, 10, 4, width = "100%")
          )
        )
      )
    )
  ),
  nav_spacer(),
  nav_item(glass_theme_toggle(selected = glass_preset))
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)

  notify <- function(type) {
    observeEvent(input[[paste0("notify_", type)]], {
      showNotification(
        paste("Notification type:", type),
        type = type,
        duration = 4
      )
    }, ignoreInit = TRUE)
  }
  notify("default")
  notify("message")
  notify("warning")
  notify("error")

  observeEvent(input$show_server_modal, {
    showModal(modalDialog(
      title = "Server modalDialog",
      easyClose = TRUE,
      footer = tagList(
        modalButton("Cancel"),
        actionButton("modal_ok", "OK", class = "btn-primary")
      ),
      selectInput(
        "modal_choice",
        "Choice",
        c("Ultra Clear", "Balanced", "Tinted"),
        selected = "Balanced",
        width = "100%"
      ),
      textInput("modal_text", "Note", "Hello from showModal()", width = "100%")
    ))
  }, ignoreInit = TRUE)

  observeEvent(input$modal_ok, {
    removeModal()
    showNotification(
      paste0("Modal OK — ", input$modal_choice %||% "?", " / ", input$modal_text %||% ""),
      type = "message",
      duration = 3
    )
  }, ignoreInit = TRUE)
}

shinyApp(ui, server)

library(shiny)
library(bslib)
library(ggplot2)
library(shinyglass)

ui <- fluidPage(
  theme = glass_theme(preset = "auto"),
  titlePanel("Liquid Glass"),
  # Min column width so cards stack on phones instead of forcing 2 columns
  layout_column_wrap(
    width = 280,
    gap = "0.75rem",
    card(
      card_header("Form Controls"),
      selectInput("species", "Species", c("setosa", "versicolor", "virginica"), width = "100%"),
      sliderInput("bins", "Bins", 1, 50, 20, width = "100%"),
      checkboxInput("smooth", "Show smooth", TRUE),
      hr(),
      glass_intensity_slider("glass_intensity"),
      glass_theme_toggle(selected = "auto")
    ),
    card(
      card_header("Plot"),
      plotOutput("dist_plot", height = "260px")
    )
  )
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)
  observe_glass_intensity(input, session, "glass_intensity")

  output$dist_plot <- renderPlot({
    x <- faithful$waiting
    df <- data.frame(x = x)
    dark <- identical(glass_resolved_preset(input), "dark")
    ink <- if (dark) "#f5f5f7" else "#1d1d1f"
    p <- ggplot(df, aes(x)) +
      geom_histogram(bins = input$bins, fill = "#007AFF", color = NA, alpha = 0.88) +
      labs(title = "Faithful Waiting Times", x = "Waiting (minutes)", y = NULL) +
      theme_minimal(base_size = 13) +
      theme(
        panel.background = element_rect(fill = NA, color = NA),
        plot.background = element_rect(fill = NA, color = NA),
        text = element_text(color = ink),
        axis.text = element_text(color = ink),
        axis.title = element_text(color = ink),
        plot.title = element_text(color = ink)
      )
    if (isTRUE(input$smooth)) {
      p <- p + geom_density(
        aes(y = after_stat(count)),
        color = "#AF52DE",
        linewidth = 1.15
      )
    }
    p
  }, bg = "transparent", height = 260, res = 96)
}

shinyApp(ui, server)

#' Liquid Glass intensity slider (iOS 27-style)
#'
#' UI control matching iOS 27 **Settings -> Appearance -> Liquid Glass**: a
#' continuous slider from **Ultra Clear** (`0`) to **Tinted** (`1`) that
#' live-updates the glass material without a page reload.
#'
#' The client applies changes immediately via `window.shinyglass.setIntensity()`.
#' The value is also a normal Shiny input (`input[[inputId]]`) so the server can
#' react or persist it. Pair with [glass_theme()] `intensity=` for the starting
#' value, and optionally [observe_glass_intensity()] to push server-driven
#' updates through [update_glass_theme()].
#'
#' @param inputId The `input` slot that will be used to access the value.
#' @param label Display label for the control (or `NULL` for none).
#' @param value Initial intensity in \eqn{[0, 1]}. If `NULL`, uses the theme
#'   default from [glass_theme()] / the client current intensity.
#' @param min,max,step Range for the underlying range input. Defaults cover
#'   the full Ultra Clear -> Tinted spectrum.
#' @param min_label,max_label End-cap captions (iOS uses "Ultra Clear" /
#'   "Tinted").
#' @param preview Show three mini glass chips as a live material sample.
#' @param width CSS width (passed to [shiny::validateCssUnit()]).
#'
#' @return A Shiny UI tag hierarchy.
#'
#' @examples
#' if (interactive()) {
#'   library(shiny)
#'   library(shinyglass)
#'
#'   ui <- fluidPage(
#'     theme = glass_theme(intensity = 0.35),
#'     glass_theme_toggle(),
#'     glass_intensity_slider("glass_intensity"),
#'     plotOutput("p")
#'   )
#'
#'   server <- function(input, output, session) {
#'     observe_glass_theme_toggle(input, session)
#'     # optional: mirror slider -> server message (client already updates live)
#'     observe_glass_intensity(input, session, "glass_intensity")
#'     output$p <- renderPlot(plot(rnorm(100), rnorm(100), pch = 16, col = "#007AFF"))
#'   }
#'
#'   shinyApp(ui, server)
#' }
#'
#' @export
glass_intensity_slider <- function(
    inputId = "glass_intensity",
    label = "Liquid Glass",
    value = NULL,
    min = 0,
    max = 1,
    step = 0.01,
    min_label = "Ultra Clear",
    max_label = "Tinted",
    preview = TRUE,
    width = NULL) {
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))
  if (!is.null(value)) {
    value <- .glass_normalize_intensity(value)
  } else {
    value <- 0.45
  }
  stopifnot(
    is.numeric(min), length(min) == 1L,
    is.numeric(max), length(max) == 1L,
    is.numeric(step), length(step) == 1L,
    min < max
  )

  style <- if (!is.null(width)) {
    sprintf("width:%s;", shiny::validateCssUnit(width))
  } else {
    NULL
  }

  chips <- if (isTRUE(preview)) {
    htmltools::tags$div(
      class = "glass-intensity-preview",
      htmltools::tags$div(class = "glass-intensity-chip", "Clear"),
      htmltools::tags$div(class = "glass-intensity-chip", "Default"),
      htmltools::tags$div(class = "glass-intensity-chip", "Tinted")
    )
  } else {
    NULL
  }

  htmltools::tags$div(
    class = "form-group shiny-input-container glass-intensity-slider",
    style = style,
    `data-glass-intensity-input` = inputId,
    if (!is.null(label)) {
      htmltools::tags$label(
        class = "control-label",
        id = paste0(inputId, "-label"),
        `for` = inputId,
        label
      )
    },
    htmltools::tags$div(
      class = "glass-intensity-row",
      htmltools::tags$span(
        class = "glass-intensity-end glass-intensity-end--min",
        min_label
      ),
      htmltools::tags$div(
        class = "glass-intensity-track",
        htmltools::tags$div(class = "glass-intensity-fill"),
        htmltools::tags$div(class = "glass-intensity-thumb", `aria-hidden` = "true"),
        htmltools::tags$input(
          type = "range",
          class = "glass-intensity-range form-range",
          id = inputId,
          min = min,
          max = max,
          step = step,
          value = value,
          `aria-valuemin` = min,
          `aria-valuemax` = max,
          `aria-valuenow` = value,
          `aria-labelledby` = if (!is.null(label)) paste0(inputId, "-label") else NULL
        )
      ),
      htmltools::tags$span(
        class = "glass-intensity-end glass-intensity-end--max",
        max_label
      )
    ),
    chips,
    htmltools::tags$span(
      class = "glass-intensity-hint",
      "Drag to match iOS 27 Liquid Glass - Ultra Clear through Tinted."
    )
  )
}

#' Keep session intensity in sync with [glass_intensity_slider()]
#'
#' The slider already updates glass live on the client. This observer optionally
#' echoes the value through [update_glass_theme()] so other clients / server
#' state stay aligned.
#'
#' @param input The server `input` object.
#' @param session A Shiny session object.
#' @param inputId Input id of the intensity slider.
#'
#' @return An [shiny::observeEvent()] observer (invisibly).
#'
#' @export
observe_glass_intensity <- function(input, session, inputId = "glass_intensity") {
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))
  shiny::observeEvent(
    input[[inputId]],
    {
      val <- input[[inputId]]
      if (is.null(val) || length(val) != 1L || is.na(val)) {
        return()
      }
      update_glass_theme(session, intensity = as.numeric(val))
    },
    ignoreInit = TRUE,
    ignoreNULL = TRUE
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(shiny)
# library(shinyglass)
# 
# ui <- fluidPage(
#   theme = glass_theme(), # or preset = "dark" / "auto"
#   titlePanel("Liquid Glass"),
#   plotOutput("p")
# )

## -----------------------------------------------------------------------------
# ui <- fluidPage(
#   theme = glass_theme(preset = "auto"),
#   glass_theme_toggle(selected = "auto"),
#   selectInput("accent", "Accent", c(
#     Blue = "#007AFF", Purple = "#AF52DE", Orange = "#FF9500"
#   ))
# )
# 
# server <- function(input, output, session) {
#   observe_glass_theme_toggle(input, session)
#   observeEvent(input$accent, {
#     update_glass_theme(session, primary = input$accent)
#   }, ignoreInit = TRUE)
# }

## -----------------------------------------------------------------------------
# ui <- fluidPage(
#   theme = glass_theme(preset = "auto"),
#   glass_theme_toggle(),           # Light / Dark / Auto buttons
#   # ...
# )
# 
# server <- function(input, output, session) {
#   observe_glass_theme_toggle(input, session)
# }

## -----------------------------------------------------------------------------
# glass_theme(
#   preset = "auto",
#   primary = "#AF52DE",
#   blur = 36,
#   saturation = 200,
#   radius = "1.5rem",
#   material = "regular", # or "clear" over media-rich content
#   intensity = 0.45,     # 0 Ultra Clear → 1 Tinted (iOS 27)
#   tint = TRUE,      # sample plot/image colors into glass surfaces
#   specular = TRUE,  # pointer specular highlight
#   nav_morph = TRUE  # compact navbar while scrolling down
# )

## -----------------------------------------------------------------------------
# glass_intensity_slider("glass_intensity", value = 0.45)

## -----------------------------------------------------------------------------
# options(teal.bs_theme = glass_theme(preset = "auto"))
# # then teal::init(...) as usual

# plotly + gt + optional waiter under glass_theme().
#
# Run:
#   shiny::runApp(system.file("examples", "plotly-gt-demo.R", package = "shinyglass"))
#
# Dark:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "plotly-gt-demo.R", package = "shinyglass"))
#
# Requires: plotly, gt  (optional: waiter)

library(shiny)
library(bslib)
library(shinyglass)

if (!requireNamespace("plotly", quietly = TRUE)) {
  stop("Install plotly: install.packages('plotly')", call. = FALSE)
}
if (!requireNamespace("gt", quietly = TRUE)) {
  stop("Install gt: install.packages('gt')", call. = FALSE)
}
library(plotly)
library(gt)

# Screenshots / CI can set SHINYGLASS_NO_WAITER=1 to skip the full-page overlay.
has_waiter <- requireNamespace("waiter", quietly = TRUE) &&
  !identical(Sys.getenv("SHINYGLASS_NO_WAITER", ""), "1")
if (has_waiter) {
  library(waiter)
}

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "auto"),
  c("light", "dark", "auto")
)

ui <- page_sidebar(
  title = "plotly + gt + glass",
  theme = glass_theme(preset = glass_preset),
  fillable = TRUE,
  if (has_waiter) use_waiter() else NULL,
  if (has_waiter) waiter_show_on_load(spin_fading_circles(), color = "rgba(0,0,0,0.35)") else NULL,
  sidebar = sidebar(
    title = "Controls",
    width = 280,
    open = "desktop",
    glass_preset_input(
      "preset",
      label = "Theme",
      selected = glass_preset,
      choices = c("Light" = "light", "Dark" = "dark", "Auto" = "auto"),
      width = "100%"
    ),
    selectInput(
      "species",
      "Species",
      c("All", levels(iris$Species)),
      selected = "All",
      width = "100%"
    ),
    actionButton("reload", "Refresh plot", class = "btn-primary", width = "100%"),
    tags$hr(),
    tags$small(
      class = "text-muted",
      if (has_waiter) {
        "waiter is loaded — a glass-tinted overlay runs on first paint and refresh."
      } else {
        "Optional: install.packages('waiter') for loading overlays."
      }
    )
  ),
  # Even split on desktop so the gt card is wide enough for full column labels
  # (a 7/5 split was clipping "Petal Length" under overflow:hidden glass).
  layout_columns(
    col_widths = breakpoints(xs = c(12, 12), md = c(6, 6)),
    gap = "0.75rem",
    card(
      full_screen = TRUE,
      card_header("plotly"),
      # Tall enough that plotly bottom margin + x-axis title fit inside the card
      plotlyOutput("scatter", height = "400px")
    ),
    card(
      full_screen = TRUE,
      card_header("gt"),
      gt_output("iris_gt")
    )
  )
)

server <- function(input, output, session) {
  if (has_waiter) {
    waiter_hide()
  }

  observe_glass_preset_input(input, session, "preset")

  filtered <- reactive({
    input$reload
    df <- iris
    if (!identical(input$species, "All")) {
      df <- df[df$Species == input$species, , drop = FALSE]
    }
    df
  })

  output$scatter <- renderPlotly({
    if (has_waiter) {
      waiter_show(html = spin_fading_circles(), color = "rgba(0,0,0,0.25)")
      on.exit(waiter_hide(), add = TRUE)
    }
    df <- filtered()
    dark <- identical(glass_resolved_preset(input), "dark")
    paper <- if (dark) "rgba(0,0,0,0)" else "rgba(255,255,255,0)"
    font_col <- if (dark) "#f5f5f7" else "#1d1d1f"
    grid_col <- if (dark) "rgba(255,255,255,0.12)" else "rgba(0,0,0,0.08)"

    plot_ly(
      df,
      x = ~Sepal.Length,
      y = ~Sepal.Width,
      color = ~Species,
      type = "scatter",
      mode = "markers",
      marker = list(size = 9, opacity = 0.85)
    ) |>
      layout(
        paper_bgcolor = paper,
        plot_bgcolor = paper,
        font = list(color = font_col, family = "system-ui, sans-serif"),
        xaxis = list(gridcolor = grid_col, zerolinecolor = grid_col, color = font_col),
        yaxis = list(gridcolor = grid_col, zerolinecolor = grid_col, color = font_col),
        legend = list(bgcolor = "rgba(0,0,0,0)", font = list(color = font_col)),
        # Room for axis titles (b especially) — tight margins clip under glass
        margin = list(l = 56, r = 24, t = 36, b = 64)
      ) |>
      config(displaylogo = FALSE, modeBarButtonsToRemove = c("lasso2d", "select2d"), responsive = TRUE)
  })

  output$iris_gt <- render_gt({
    df <- filtered()
    summary <- aggregate(
      cbind(Sepal.Length, Sepal.Width, Petal.Length) ~ Species,
      data = df,
      FUN = function(x) round(mean(x), 2)
    )
    # Leave font color unset so glass CSS can apply --glass-body-color
    # (gt rejects CSS keyword "inherit" as a color name).
    gt(summary) |>
      tab_header(
        title = "Mean iris measures",
        subtitle = paste(nrow(df), "rows")
      ) |>
      cols_label(
        Species = "Species",
        Sepal.Length = "Sepal Length",
        Sepal.Width = "Sepal Width",
        Petal.Length = "Petal Length"
      ) |>
      cols_align(align = "left", columns = Species) |>
      cols_align(align = "right", columns = where(is.numeric)) |>
      cols_width(
        Species ~ px(96),
        Sepal.Length ~ px(104),
        Sepal.Width ~ px(104),
        Petal.Length ~ px(104)
      ) |>
      opt_row_striping() |>
      tab_options(
        table.width = pct(100),
        table.background.color = "transparent",
        heading.background.color = "transparent",
        column_labels.background.color = "transparent",
        row.striping.background_color = "transparent",
        table.border.top.color = "transparent",
        table.border.bottom.color = "transparent",
        table.border.left.color = "transparent",
        table.border.right.color = "transparent",
        column_labels.padding = "8px",
        data_row.padding = "6px"
      )
  })
}

shinyApp(ui, server)

#' Liquid Glass Design Themes for 'shiny' Applications
#'
#' Drop-in Liquid Glass themes for [shiny](https://shiny.posit.co/). Call
#' [glass_theme()] and pass the result to `theme =` on `fluidPage()`,
#' `navbarPage()`, or any [bslib](https://rstudio.github.io/bslib/)-aware page
#' function to get translucent surfaces, backdrop blur, and system typography.
#'
#' @section Getting started:
#' ```r
#' library(shiny)
#' library(shinyglass)
#'
#' ui <- fluidPage(
#'   theme = glass_theme(preset = "auto", intensity = 0.45),
#'   titlePanel("Liquid Glass"),
#'   glass_intensity_slider("glass_intensity"),
#'   glass_theme_toggle(selected = "auto"),
#'   sliderInput("n", "Bars", 5, 30, 15),
#'   plotOutput("plot")
#' )
#'
#' server <- function(input, output, session) {
#'   observe_glass_theme_toggle(input, session)
#'   observe_glass_intensity(input, session, "glass_intensity")
#'   output$plot <- renderPlot(
#'     barplot(seq_len(input$n), col = "#007AFF", border = NA)
#'   )
#' }
#' ```
#'
#' Light and dark presets are available via `glass_theme(preset = "dark")`,
#' or `preset = "auto"` to follow the OS. Switch at runtime with
#' [update_glass_theme()] or [glass_theme_toggle()]. Material density is
#' controlled with [glass_intensity_slider()] (Ultra Clear to Tinted) or
#' `glass_theme(intensity = )`. Accent color, blur, saturation, corner radius,
#' and JS behaviors (`tint`, `specular`, `nav_morph`) are configurable.
#'
#' For [teal](https://insightsengineering.github.io/teal/) apps, set
#' `options(teal.bs_theme = glass_theme())` before calling `teal::init()`.
#'
#' @seealso [glass_theme()], [glass_intensity_slider()], [update_glass_theme()],
#'   [glass_theme_toggle()]
#'
#' @keywords internal
"_PACKAGE"

# Modern bslib dashboard: value boxes, layout_columns, navset_card_tab.
#
# Run:
#   shiny::runApp(system.file("examples", "bslib-dashboard.R", package = "shinyglass"))
#
# Dark preset:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "bslib-dashboard.R", package = "shinyglass"))

library(shiny)
library(bslib)
library(shinyglass)

if (!requireNamespace("ggplot2", quietly = TRUE)) {
  stop("ggplot2 is required for this demo. Install with install.packages('ggplot2').")
}
if (!requireNamespace("DT", quietly = TRUE)) {
  stop("DT is required for this demo. Install with install.packages('DT').")
}
library(ggplot2)

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "auto"),
  c("light", "dark", "auto")
)

accent_colors <- c(
  "Apple Blue" = "#007AFF",
  "Purple" = "#AF52DE",
  "Orange" = "#FF9500",
  "Green" = "#34C759"
)

ui <- page_sidebar(
  title = "Glass Dashboard",
  theme = glass_theme(preset = glass_preset, primary = "#007AFF", intensity = 0.45),
  class = "bslib-page-dashboard",
  fillable = TRUE,
  # Closed on phones by default so content is usable; open on desktop
  sidebar = sidebar(
    title = "Dashboard",
    width = 280,
    open = "desktop",
    glass_intensity_slider(
      "glass_intensity",
      label = "Liquid Glass",
      value = 0.45,
      preview = TRUE
    ),
    glass_preset_input(
      "preset",
      label = "Theme preset",
      selected = glass_preset,
      width = "100%"
    ),
    selectInput(
      "accent",
      "Accent color",
      choices = accent_colors,
      selected = "#007AFF",
      width = "100%"
    ),
    selectInput(
      "species",
      "Focus species",
      choices = c("All", "setosa", "versicolor", "virginica"),
      selected = "All",
      width = "100%"
    ),
    sliderInput("bins", "Histogram bins", 8, 40, 18, width = "100%"),
    checkboxInput("show_curve", "Show density curve", TRUE),
    actionButton("refresh", "Refresh metrics", class = "btn-primary", width = "100%")
  ),
  # Stack value boxes on narrow screens (min width ~10rem)
  layout_column_wrap(
    width = "10rem",
    fill = FALSE,
    gap = "0.65rem",
    value_box(
      title = "Observations",
      value = textOutput("metric_n"),
      theme = "primary"
    ),
    value_box(
      title = "Species",
      value = textOutput("metric_species"),
      theme = "success"
    ),
    value_box(
      title = "Avg sepal length",
      value = textOutput("metric_sepal"),
      theme = "info"
    )
  ),
  # Full-width stack below md; two columns from md up
  layout_columns(
    col_widths = breakpoints(
      xs = c(12, 12),
      md = c(6, 6)
    ),
    gap = "0.75rem",
    navset_card_tab(
      id = "tabs",
      nav_panel(
        "Distribution",
        card_body(
          plotOutput("dist_plot", height = "280px")
        )
      ),
      nav_panel(
        "Scatter",
        card_body(
          plotOutput("scatter_plot", height = "280px")
        )
      )
    ),
    card(
      full_screen = TRUE,
      card_header("Iris data"),
      DT::DTOutput("data_table")
    )
  )
)

server <- function(input, output, session) {
  filtered_data <- reactive({
    input$refresh
    df <- iris
    if (input$species != "All") {
      df <- df[df$Species == input$species, , drop = FALSE]
    }
    df
  })

  observe_glass_intensity(input, session, "glass_intensity")
  observe_glass_preset_input(input, session, "preset")

  observeEvent(input$accent, {
    update_glass_theme(session, primary = input$accent)
  }, ignoreInit = TRUE)

  output$metric_n <- renderText({
    format(nrow(filtered_data()), big.mark = ",")
  })

  output$metric_species <- renderText({
    length(unique(filtered_data()$Species))
  })

  output$metric_sepal <- renderText({
    avg <- mean(filtered_data()$Sepal.Length, na.rm = TRUE)
    paste0(round(avg, 1), " cm")
  })

  plot_fg <- reactive({
    if (identical(glass_resolved_preset(input), "dark")) "#f5f5f7" else "#1d1d1f"
  })

  glass_gg <- function() {
    theme_minimal(base_size = 12) +
      theme(
        panel.background = element_rect(fill = NA, color = NA),
        plot.background = element_rect(fill = NA, color = NA),
        legend.background = element_rect(fill = NA, color = NA),
        legend.box.background = element_rect(fill = NA, color = NA),
        text = element_text(color = plot_fg()),
        axis.text = element_text(color = plot_fg()),
        plot.title = element_text(color = plot_fg()),
        legend.text = element_text(color = plot_fg()),
        legend.title = element_text(color = plot_fg())
      )
  }

  output$dist_plot <- renderPlot({
    df <- filtered_data()
    accent <- input$accent
    p <- ggplot(df, aes(x = Sepal.Length)) +
      geom_histogram(bins = input$bins, fill = accent, color = NA, alpha = 0.85) +
      labs(title = "Sepal length distribution", x = NULL, y = "Count") +
      glass_gg()
    if (isTRUE(input$show_curve)) {
      p <- p + geom_density(
        aes(y = after_stat(count)),
        color = accent,
        linewidth = 1,
        alpha = 0.5
      )
    }
    print(p)
  }, bg = "transparent", height = 280, res = 96)

  output$scatter_plot <- renderPlot({
    df <- filtered_data()
    accent <- input$accent
    ggplot(df, aes(x = Sepal.Length, y = Sepal.Width, color = Species)) +
      geom_point(size = 2.4, alpha = 0.85) +
      scale_color_manual(values = c("#007AFF", "#AF52DE", "#FF9500")) +
      labs(title = "Sepal dimensions", x = "Length", y = "Width") +
      glass_gg() +
      theme(legend.position = "bottom")
  }, bg = "transparent", height = 280, res = 96)

  output$data_table <- DT::renderDT({
    DT::datatable(
      filtered_data(),
      fillContainer = FALSE,
      rownames = FALSE,
      options = list(
        pageLength = 6,
        dom = "tip",
        scrollX = TRUE,
        autoWidth = TRUE
      )
    )
  })
}

shinyApp(ui, server)

#' Liquid Glass theme for 'shiny'
#'
#' Create a [bslib::bs_theme()] styled with a Liquid Glass look: translucent
#' surfaces, backdrop blur, soft depth, and system typography. Pass the result
#' to `theme =` on `fluidPage()`, `navbarPage()`, `bslib::page_sidebar()`, or
#' any other page function that accepts a bslib theme.
#'
#' Light and dark surface tokens are compiled into dual CSS custom-property
#' packs. Switching `preset` at runtime (via [update_glass_theme()] or
#' `preset = "auto"`) updates `document.documentElement.dataset.glassPreset`
#' without recompiling Sass or reloading the page. Accent color can also be
#' updated live with [update_glass_theme()] `primary=` (CSS variables).
#'
#' @param preset `"light"`, `"dark"`, or `"auto"`. `"auto"` follows
#'   `prefers-color-scheme` and updates when the OS theme changes.
#' @param primary Accent color for buttons, links, and focus rings.
#'   Defaults to system blue (`#007AFF`).
#' @param blur Backdrop blur radius in pixels. Default `36` matches the
#'   iOS 27 diffusion-first material.
#' @param saturation Backdrop saturation percentage.
#' @param radius Default border radius for glass surfaces (CSS length).
#'   Prefer larger concentric radii (default `1.5rem`).
#' @param material `"regular"` (adaptive, most UI) or `"clear"` (more
#'   transparent; best over media-rich content with bold labels).
#' @param intensity Liquid Glass intensity from `0` (Ultra Clear) to `1`
#'   (Tinted), matching iOS 27 Appearance -> Liquid Glass. Default `0.45`.
#'   Use [glass_intensity_slider()] for a live control.
#' @param tint Content-aware ambient tint from plots/images (JS).
#' @param specular Pointer-driven specular highlight on glass surfaces (JS).
#' @param nav_morph Compact navbar on scroll down; expand on scroll up (JS).
#' @param ... Additional arguments forwarded to [bslib::bs_theme()].
#'
#' @return A [bslib::bs_theme()] object suitable for 'shiny' page functions.
#'
#' @examples
#' theme <- glass_theme()
#' dark <- glass_theme(preset = "dark", primary = "#BF5AF2")
#' auto <- glass_theme(preset = "auto", tint = FALSE)
#' clear <- glass_theme(material = "clear")
#'
#' if (interactive()) {
#'   library(shiny)
#'
#'   ui <- fluidPage(
#'     theme = glass_theme(preset = "auto"),
#'     titlePanel("Liquid Glass"),
#'     glass_theme_toggle(),
#'     selectInput("color", "Color", c("Blue", "Purple", "Orange")),
#'     plotOutput("plot")
#'   )
#'
#'   server <- function(input, output, session) {
#'     # Client onclick already switches; keep session in sync:
#'     observe_glass_theme_toggle(input, session)
#'   }
#'
#'   shinyApp(ui, server)
#' }
#'
#' @export
glass_theme <- function(
    preset = c("light", "dark", "auto"),
    primary = "#007AFF",
    blur = 36,
    saturation = 200,
    radius = "1.5rem",
    material = c("regular", "clear"),
    intensity = 0.45,
    tint = TRUE,
    specular = TRUE,
    nav_morph = TRUE,
    ...) {
  preset <- match.arg(preset)
  material <- match.arg(material)
  intensity <- .glass_normalize_intensity(intensity)
  stopifnot(
    is.logical(tint), length(tint) == 1L, !is.na(tint),
    is.logical(specular), length(specular) == 1L, !is.na(specular),
    is.logical(nav_morph), length(nav_morph) == 1L, !is.na(nav_morph)
  )
  primary <- .glass_normalize_color(primary)

  # Sass still needs a single pack of $glass-* defaults at compile time.
  # Runtime light/dark comes from dual CSS variable packs in glass.scss.
  # Always use the Bootstrap base (not darkly) so switching preset does not
  # fight Bootswatch dark chrome.
  tokens <- .glass_tokens("light", blur, saturation, radius)

  theme <- bslib::bs_theme(
    version = 5,
    preset = "bootstrap",
    primary = primary,
    "body-bg" = tokens$body_bg,
    "body-color" = tokens$body_color,
    "font-family-sans-serif" = .glass_font_stack(),
    "border-radius" = "1.1rem",
    "border-radius-lg" = radius,
    "border-radius-sm" = "0.85rem",
    "card-border-width" = "1px",
    "card-border-color" = tokens$glass_border,
    "input-border-color" = tokens$glass_border,
    "navbar-padding-y" = "0.75rem",
    "btn-font-weight" = 600,
    "btn-font-size" = "0.9375rem",
    "btn-line-height" = 1.2,
    "btn-padding-y" = ".55rem",
    "btn-padding-x" = "1.2rem",
    "btn-border-width" = "1px",
    # Bootstrap color-contrast() defaults to WCAG AA 4.5:1. White on system
    # blue #007AFF is ~4.02:1, so the gate picks black for buttons, text-bg-*,
    # active states, etc. 3:1 is WCAG UI-component minimum and lets white win
    # on brand blues/purples while pastels (info cyan, warning yellow) still
    # get dark ink. glass.scss glass-on() + solid .bg-* rules are the CSS
    # safety net for value boxes (which use .bg-primary without text color).
    "min-contrast-ratio" = 3,
    # Explicit active-on-accent ink (SVG checks/switches/pagination).
    "component-active-color" = "#ffffff",
    "form-check-input-checked-color" = "#ffffff",
    "form-check-input-indeterminate-color" = "#ffffff",
    "form-switch-checked-color" = "#ffffff",
    "pagination-active-color" = "#ffffff",
    ...
  )

  theme <- bslib::bs_add_variables(
    theme,
    # Shared knobs still used by Sass ($glass-blur, etc.)
    "glass-bg" = tokens$glass_bg,
    "glass-bg-hover" = tokens$glass_bg_hover,
    "glass-border" = tokens$glass_border,
    "glass-shadow" = tokens$glass_shadow,
    "glass-elevated-shadow" = tokens$glass_elevated_shadow,
    "glass-blur" = paste0(blur, "px"),
    "glass-saturate" = paste0(saturation, "%"),
    "glass-radius" = radius,
    "glass-highlight" = tokens$glass_highlight,
    "glass-specular" = tokens$glass_specular,
    "glass-menu-bg" = tokens$glass_menu_bg,
    "glass-menu-color" = tokens$glass_menu_color,
    "glass-page-bg" = tokens$page_bg,
    "glass-orb-1" = tokens$orb_1,
    "glass-orb-2" = tokens$orb_2,
    "glass-orb-3" = tokens$orb_3
  )

  glass_scss <- system.file("scss", "glass.scss", package = "shinyglass")
  theme <- bslib::bs_add_rules(theme, sass::sass_file(glass_scss))

  pkg_version <- as.character(utils::packageVersion("shinyglass"))
  js_src <- system.file("js", package = "shinyglass")

  # Early head script: set data-glass-preset before first paint.
  # "auto" resolves prefers-color-scheme on the client.
  head_script <- .glass_preset_head_script(
    preset = preset,
    tint = tint,
    specular = specular,
    nav_morph = nav_morph,
    primary = primary,
    material = material,
    intensity = intensity
  )

  # htmlDependency (not tagFunction-returned tags) so htmltools does not
  # warn when dependencies are resolved via bs_theme_dependencies().
  preset_dep <- htmltools::htmlDependency(
    name = "shinyglass-preset",
    version = pkg_version,
    src = js_src,
    head = head_script,
    all_files = FALSE
  )

  glass_js <- htmltools::htmlDependency(
    name = "shinyglass",
    version = pkg_version,
    src = js_src,
    script = "shiny-glass.js",
    all_files = FALSE
  )
  bslib::bs_bundle(
    theme,
    sass::sass_layer(html = preset_dep),
    sass::sass_layer(html = glass_js)
  )
}

#' Update glass theme options in a running app
#'
#' Send a message to the browser to change the Liquid Glass preset, accent
#' color, intensity, or content-tint behavior without reloading the page.
#' Requires a page that used [glass_theme()] (so `shiny-glass.js` is loaded).
#'
#' `primary` updates CSS variables (`--bs-primary`, `--bs-primary-rgb`,
#' `--glass-primary`) so buttons, checks, and other accent surfaces follow the
#' new color. Sass-baked one-off colors may not all switch until a full reload.
#' `intensity` updates fill/blur along the Ultra Clear to Tinted spectrum
#' (see [glass_intensity_slider()]).
#'
#' @param session A Shiny session object (usually the `session` argument of
#'   the server function).
#' @param preset Optional. `"light"`, `"dark"`, or `"auto"`.
#' @param tint Optional logical. Enable or disable content-aware ambient tint.
#' @param primary Optional accent color (hex like `"#AF52DE"` or `rgb()`).
#' @param intensity Optional numeric in \eqn{[0, 1]}: Ultra Clear (`0`) to
#'   Tinted (`1`).
#'
#' @return `session`, invisibly.
#'
#' @examples
#' if (interactive()) {
#'   library(shiny)
#'   library(shinyglass)
#'
#'   ui <- fluidPage(
#'     theme = glass_theme(),
#'     glass_theme_toggle(),
#'     selectInput("accent", "Accent", c("#007AFF", "#AF52DE", "#FF9500"))
#'   )
#'
#'   server <- function(input, output, session) {
#'     observe_glass_theme_toggle(input, session)
#'     observeEvent(input$accent, {
#'       update_glass_theme(session, primary = input$accent)
#'     }, ignoreInit = TRUE)
#'   }
#'
#'   shinyApp(ui, server)
#' }
#'
#' @export
update_glass_theme <- function(
    session,
    preset = NULL,
    tint = NULL,
    primary = NULL,
    intensity = NULL) {
  if (missing(session) || is.null(session)) {
    stop("`session` is required.", call. = FALSE)
  }
  if (!is.null(preset)) {
    preset <- match.arg(preset, c("light", "dark", "auto"))
  }
  if (!is.null(tint)) {
    stopifnot(is.logical(tint), length(tint) == 1L, !is.na(tint))
  }
  if (!is.null(primary)) {
    primary <- .glass_normalize_color(primary)
  }
  if (!is.null(intensity)) {
    intensity <- .glass_normalize_intensity(intensity)
  }
  if (is.null(preset) && is.null(tint) && is.null(primary) && is.null(intensity)) {
    return(invisible(session))
  }

  payload <- list()
  if (!is.null(preset)) payload$preset <- preset
  if (!is.null(tint)) payload$tint <- tint
  if (!is.null(primary)) payload$primary <- primary
  if (!is.null(intensity)) payload$intensity <- intensity

  # Dual-channel delivery: structured shinyglass payload + legacy glassPreset
  # string. Some hosts rewrite or drop one channel but not both; the client
  # also applies user-driven preset/intensity changes immediately (see JS).
  session$sendCustomMessage("shinyglass", payload)
  if (!is.null(preset)) {
    session$sendCustomMessage("glassPreset", preset)
  }
  invisible(session)
}

#' Light / dark / auto theme toggle buttons
#'
#' Drop-in button group for switching the glass preset. Each button sets the
#' preset on the client immediately (`window.shinyglass.setPreset`) and also
#' has a 'shiny' input id so [observe_glass_theme_toggle()] can keep the server
#' in sync (important on hosts that rewrite custom messages).
#'
#' @param inputId Base id. Buttons are `{inputId}_light`, `{inputId}_dark`,
#'   and `{inputId}_auto`.
#' @param selected Initially highlighted mode (`"light"`, `"dark"`, or
#'   `"auto"`). Cosmetic only; the page theme still comes from [glass_theme()].
#' @param labels Named character vector for button labels. Names must be
#'   `light`, `dark`, and/or `auto`.
#' @param class Extra CSS classes for the wrapper.
#'
#' @return An [htmltools::tag()] button group.
#'
#' @seealso [observe_glass_theme_toggle()], [update_glass_theme()]
#'
#' @export
glass_theme_toggle <- function(
    inputId = "glass_toggle",
    selected = c("auto", "light", "dark"),
    labels = c(light = "Light", dark = "Dark", auto = "Auto (OS)"),
    class = "glass-theme-toggle") {
  selected <- match.arg(selected)
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))
  modes <- c("light", "dark", "auto")
  if (is.null(names(labels)) || !all(modes %in% names(labels))) {
    stop("`labels` must be a named vector with names light, dark, auto.", call. = FALSE)
  }

  btns <- lapply(modes, function(mode) {
    id <- paste0(inputId, "_", mode)
    active <- identical(mode, selected)
    btn_class <- if (active) "btn-sm btn-primary" else "btn-sm btn-secondary"
    shiny::actionButton(
      inputId = id,
      label = unname(labels[[mode]]),
      class = btn_class,
      `data-glass-preset-mode` = mode,
      `aria-pressed` = if (active) "true" else "false"
    )
  })

  htmltools::div(
    class = class,
    role = "group",
    `aria-label` = "Glass theme preset",
    btns
  )
}

#' Observe [glass_theme_toggle()] buttons on the server
#'
#' Wires the three toggle inputs to [update_glass_theme()] so session state
#' stays aligned with the client (needed on some hosts that rewrite 'shiny'
#' messaging).
#'
#' @param input The server `input` object.
#' @param session The server `session` object.
#' @param inputId Same base id passed to [glass_theme_toggle()].
#'
#' @return `NULL`, invisibly. Called for side effects (registers observers).
#'
#' @export
observe_glass_theme_toggle <- function(input, session, inputId = "glass_toggle") {
  if (missing(input) || missing(session)) {
    stop("`input` and `session` are required.", call. = FALSE)
  }
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))

  shiny::observeEvent(
    input[[paste0(inputId, "_light")]],
    update_glass_theme(session, preset = "light"),
    ignoreInit = TRUE
  )
  shiny::observeEvent(
    input[[paste0(inputId, "_dark")]],
    update_glass_theme(session, preset = "dark"),
    ignoreInit = TRUE
  )
  shiny::observeEvent(
    input[[paste0(inputId, "_auto")]],
    update_glass_theme(session, preset = "auto"),
    ignoreInit = TRUE
  )
  invisible(NULL)
}

#' Light / dark / auto preset select input
#'
#' Drop-in [shiny::selectInput()] for the glass theme preset. The client applies
#' the change **immediately** via `window.shinyglass.setPreset()` (marked with
#' `data-glass-preset-input`), so Light/Dark/Auto works even when custom
#' messages are delayed or rewritten (e.g. on some shinyapps.io hosts). Pair
#' with [observe_glass_preset_input()] so the server stays in sync.
#'
#' @param inputId The `input` slot that will be used to access the value.
#' @param label Display label for the control (or `NULL` for none).
#' @param selected Initially selected mode (`"light"`, `"dark"`, or `"auto"`).
#' @param choices Named character vector of choices. Defaults to Light / Dark /
#'   Auto (OS). Names are labels; values must be `light`, `dark`, and/or `auto`.
#' @param width The width of the input (e.g. `"100%"`).
#'
#' @return A [shiny::selectInput()] tag (with glass client bindings).
#'
#' @seealso [observe_glass_preset_input()], [glass_theme_toggle()],
#'   [update_glass_theme()]
#'
#' @export
glass_preset_input <- function(
    inputId = "glass_preset",
    label = "Theme preset",
    selected = c("auto", "light", "dark"),
    choices = c("Light" = "light", "Dark" = "dark", "Auto (OS)" = "auto"),
    width = NULL) {
  selected <- match.arg(selected)
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))
  vals <- unname(choices)
  if (!all(vals %in% c("light", "dark", "auto"))) {
    stop(
      "`choices` values must be a subset of 'light', 'dark', 'auto'.",
      call. = FALSE
    )
  }
  if (!selected %in% vals) {
    stop("`selected` must be one of the `choices` values.", call. = FALSE)
  }

  sel <- shiny::selectInput(
    inputId = inputId,
    label = label,
    choices = choices,
    selected = selected,
    width = width
  )

  # Mark the widget so shiny-glass.js applies setPreset on change (client-first).
  # Prefer tagging the <select>; fall back to the outer container.
  if (requireNamespace("htmltools", quietly = TRUE) &&
      exists("tagQuery", where = asNamespace("htmltools"), inherits = FALSE)) {
    sel <- htmltools::tagQuery(sel)$
      find("select")$
      addClass("glass-preset-input")$
      addAttrs(`data-glass-preset-input` = inputId)$
      allTags()
  } else if (is.list(sel) && !is.null(sel$attribs)) {
    sel$attribs$`data-glass-preset-input` <- inputId
    sel$attribs$class <- paste(sel$attribs$class %||% "", "glass-preset-input-wrap")
  }

  sel
}

`%||%` <- function(x, y) {
  if (is.null(x) || length(x) == 0L || (is.character(x) && !nzchar(x[[1]]))) y else x
}

#' Observe [glass_preset_input()] on the server
#'
#' Wires a preset select to [update_glass_theme()] so session state stays
#' aligned with the client (the client already applied the preset live).
#'
#' @param input The server `input` object.
#' @param session The server `session` object.
#' @param inputId Same id passed to [glass_preset_input()].
#'
#' @return An [shiny::observeEvent()] observer (invisibly).
#'
#' @export
observe_glass_preset_input <- function(input, session, inputId = "glass_preset") {
  if (missing(input) || missing(session)) {
    stop("`input` and `session` are required.", call. = FALSE)
  }
  stopifnot(is.character(inputId), length(inputId) == 1L, nzchar(inputId))
  shiny::observeEvent(
    input[[inputId]],
    {
      val <- input[[inputId]]
      if (is.null(val) || !nzchar(val) || !val %in% c("light", "dark", "auto")) {
        return()
      }
      update_glass_theme(session, preset = val)
    },
    ignoreInit = TRUE,
    ignoreNULL = TRUE
  )
}

#' Resolved light or dark appearance
#'
#' When the user picks **Auto**, `input$preset` stays `"auto"` so checks like
#' `identical(input$preset, "dark")` stay FALSE and plots keep light ink on a
#' dark page. The client publishes the *resolved* appearance as
#' `input$glass_resolved_preset` (`"light"` or `"dark"`).
#'
#' @param input The server `input` object.
#' @param default Fallback if the client has not reported yet (`"light"` or
#'   `"dark"`).
#'
#' @return `"light"` or `"dark"`.
#'
#' @export
glass_resolved_preset <- function(input, default = c("light", "dark")) {
  default <- match.arg(default)
  if (missing(input) || is.null(input)) {
    return(default)
  }
  v <- input[["glass_resolved_preset"]]
  if (length(v) == 1L && !is.na(v) && v %in% c("light", "dark")) {
    return(as.character(v))
  }
  for (key in c("preset", "glass_preset")) {
    p <- input[[key]]
    if (length(p) == 1L && !is.na(p) && p %in% c("light", "dark")) {
      return(as.character(p))
    }
  }
  default
}

.glass_font_stack <- function() {
  paste(
    "-apple-system",
    "BlinkMacSystemFont",
    '"SF Pro Display"',
    '"SF Pro Text"',
    '"Segoe UI"',
    "Roboto",
    "Helvetica",
    "Arial",
    "sans-serif",
    sep = ", "
  )
}

.glass_normalize_intensity <- function(intensity) {
  stopifnot(is.numeric(intensity), length(intensity) == 1L, !is.na(intensity))
  if (intensity < 0 || intensity > 1) {
    stop("`intensity` must be between 0 (Ultra Clear) and 1 (Tinted).", call. = FALSE)
  }
  as.numeric(intensity)
}

.glass_normalize_color <- function(primary) {
  stopifnot(is.character(primary), length(primary) == 1L, !is.na(primary))
  primary <- trimws(primary)
  if (!nzchar(primary)) {
    stop("`primary` must be a non-empty color string.", call. = FALSE)
  }
  # Accept #RGB, #RRGGBB, or rgb()/rgba() - leave other CSS colors to the browser.
  if (grepl("^#([0-9A-Fa-f]{3}|[0-9A-Fa-f]{6})$", primary) ||
    grepl("^rgba?\\(", primary, ignore.case = TRUE)) {
    return(primary)
  }
  # Named colors / other CSS - still pass through for flexibility
  primary
}

# Luminance-based ink for solid theme fills (matches glass-on() in SCSS).
# Light labels on saturated brand colors; dark on pastels.
.glass_on_color <- function(color, light = "#ffffff", dark = "#1d1d1f") {
  rgb <- .glass_hex_to_rgb(color)
  if (is.null(rgb)) {
    return(light)
  }
  y <- (0.2126 * rgb$r + 0.7152 * rgb$g + 0.0722 * rgb$b) / 255
  if (y >= 0.55) dark else light
}

.glass_tokens <- function(preset, blur, saturation, radius) {
  if (preset == "light") {
    list(
      body_bg = "#f2f2f7",
      body_color = "#1d1d1f",
      glass_bg = "rgba(255, 255, 255, 0.22)",
      glass_bg_hover = "rgba(255, 255, 255, 0.36)",
      glass_border = "rgba(255, 255, 255, 0.62)",
      glass_shadow = "rgba(0, 0, 0, 0.10)",
      glass_elevated_shadow = "rgba(0, 0, 0, 0.16)",
      glass_highlight = "rgba(255, 255, 255, 0.88)",
      glass_specular = "rgba(255, 255, 255, 0.55)",
      glass_menu_bg = "rgba(255, 255, 255, 0.86)",
      glass_menu_color = "#1d1d1f",
      page_bg = "linear-gradient(160deg, #eef1f8 0%, #f5f5f7 42%, #ebe8f4 78%, #e6eef8 100%)",
      orb_1 = "rgba(0, 122, 255, 0.18)",
      orb_2 = "rgba(175, 82, 222, 0.12)",
      orb_3 = "rgba(90, 200, 250, 0.14)"
    )
  } else {
    list(
      body_bg = "#000000",
      body_color = "#f5f5f7",
      glass_bg = "rgba(255, 255, 255, 0.14)",
      glass_bg_hover = "rgba(255, 255, 255, 0.22)",
      glass_border = "rgba(255, 255, 255, 0.26)",
      glass_shadow = "rgba(0, 0, 0, 0.48)",
      glass_elevated_shadow = "rgba(0, 0, 0, 0.62)",
      glass_highlight = "rgba(255, 255, 255, 0.22)",
      glass_specular = "rgba(255, 255, 255, 0.18)",
      glass_menu_bg = "rgba(44, 44, 46, 0.88)",
      glass_menu_color = "#f5f5f7",
      page_bg = "linear-gradient(160deg, #0a0a12 0%, #000000 45%, #100a16 80%, #060a14 100%)",
      orb_1 = "rgba(10, 132, 255, 0.28)",
      orb_2 = "rgba(191, 90, 242, 0.20)",
      orb_3 = "rgba(100, 210, 255, 0.16)"
    )
  }
}

.glass_preset_head_script <- function(
    preset,
    tint,
    specular,
    nav_morph,
    primary = "#007AFF",
    material = "regular",
    intensity = 0.45) {
  # Inline early so first paint uses the right pack. Keep this free of
  # external deps (runs before shiny-glass.js).
  rgb <- .glass_hex_to_rgb(primary)
  on_primary <- .glass_on_color(primary)
  rgb_css <- if (is.null(rgb)) {
    "null"
  } else {
    sprintf("{r:%d,g:%d,b:%d}", rgb$r, rgb$g, rgb$b)
  }
  material <- if (identical(material, "clear")) "clear" else "regular"
  intensity <- .glass_normalize_intensity(intensity)
  sprintf(
    paste0(
      "<script>(function(){",
      "var p=%s;",
      "var root=document.documentElement;",
      "root.dataset.glassMode=p;",
      "function resolve(mode){",
      "if(mode==='auto'){",
      "try{",
      "return window.matchMedia('(prefers-color-scheme: dark)').matches?'dark':'light';",
      "}catch(e){return 'light';}",
      "}",
      "return mode==='dark'?'dark':'light';",
      "}",
      "root.dataset.glassPreset=resolve(p);",
      "root.dataset.glassMaterial=%s;",
      "root.dataset.glassIntensity=%s;",
      "root.style.setProperty('--glass-intensity',%s);",
      "root.dataset.glassTint=%s;",
      "root.dataset.glassSpecular=%s;",
      "root.dataset.glassNavMorph=%s;",
      "var prim=%s;",
      "var rgb=%s;",
      "var onPrim=%s;",
      "if(prim){root.dataset.glassPrimary=prim;",
      "root.style.setProperty('--bs-primary',prim);",
      "root.style.setProperty('--glass-primary',prim);",
      "root.style.setProperty('--glass-accent',prim);",
      "root.style.setProperty('--glass-on-primary',onPrim);",
      "if(rgb){root.style.setProperty('--bs-primary-rgb',rgb.r+', '+rgb.g+', '+rgb.b);}",
      "}",
      "})();</script>"
    ),
    jsonlite_quote(preset),
    jsonlite_quote(material),
    sprintf("%.4f", intensity),
    sprintf("%.4f", intensity),
    if (isTRUE(tint)) "\"true\"" else "\"false\"",
    if (isTRUE(specular)) "\"true\"" else "\"false\"",
    if (isTRUE(nav_morph)) "\"true\"" else "\"false\"",
    jsonlite_quote(primary),
    rgb_css,
    jsonlite_quote(on_primary)
  )
}

.glass_hex_to_rgb <- function(color) {
  color <- trimws(as.character(color))
  if (grepl("^#([0-9A-Fa-f]{3})$", color)) {
    h <- substring(color, 2)
    h <- paste0(
      substr(h, 1, 1), substr(h, 1, 1),
      substr(h, 2, 2), substr(h, 2, 2),
      substr(h, 3, 3), substr(h, 3, 3)
    )
    color <- paste0("#", h)
  }
  if (!grepl("^#([0-9A-Fa-f]{6})$", color)) {
    return(NULL)
  }
  list(
    r = strtoi(substr(color, 2, 3), 16L),
    g = strtoi(substr(color, 4, 5), 16L),
    b = strtoi(substr(color, 6, 7), 16L)
  )
}

# Avoid depending on jsonlite just to quote a short string.
jsonlite_quote <- function(x) {
  paste0('"', gsub("([\\\\\"])", "\\\\\\1", as.character(x)), '"')
}

test_that("glass_theme returns a bs_theme object", {
  skip_if_not_installed("bslib")
  theme <- glass_theme()
  expect_s3_class(theme, "bs_theme")
})

test_that("glass_theme supports light, dark, and auto presets", {
  skip_if_not_installed("bslib")
  light <- glass_theme(preset = "light")
  dark <- glass_theme(preset = "dark")
  auto <- glass_theme(preset = "auto")
  expect_s3_class(light, "bs_theme")
  expect_s3_class(dark, "bs_theme")
  expect_s3_class(auto, "bs_theme")
})

test_that("glass_theme compiles dependencies", {
  skip_if_not_installed("bslib")
  theme <- glass_theme()
  deps <- bslib::bs_theme_dependencies(theme)
  expect_true(length(deps) >= 1)
  dep_names <- vapply(deps, function(d) d$name, character(1))
  expect_true("shinyglass" %in% dep_names)
  expect_true("shinyglass-preset" %in% dep_names)
})

test_that("glass_theme sets preset data attributes in head", {
  skip_if_not_installed("bslib")
  dark <- glass_theme(preset = "dark", tint = FALSE, specular = FALSE, nav_morph = FALSE)
  deps <- bslib::bs_theme_dependencies(dark)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  head <- preset_deps[[1]]$head
  expect_match(head, 'var p="dark"')
  expect_match(head, "dataset\\.glassMode")
  expect_match(head, "dataset\\.glassPreset")
  expect_match(head, 'glassTint="false"')
  expect_match(head, 'glassSpecular="false"')
  expect_match(head, 'glassNavMorph="false"')
})

test_that("glass_theme auto preset is marked in head", {
  skip_if_not_installed("bslib")
  auto <- glass_theme(preset = "auto")
  deps <- bslib::bs_theme_dependencies(auto)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  expect_match(preset_deps[[1]]$head, 'var p="auto"')
})

test_that("glass_theme material clear is marked in head and CSS", {
  skip_if_not_installed("bslib")
  th <- glass_theme(material = "clear")
  deps <- bslib::bs_theme_dependencies(th)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  expect_match(preset_deps[[1]]$head, 'glassMaterial="clear"')

  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")
  expect_match(css, "data-glass-material")
  expect_match(css, "--glass-bg-content")
  expect_match(css, "--glass-rim")
})
test_that("compiled CSS ships dual light/dark variable packs", {
  skip_if_not_installed("bslib")
  theme <- glass_theme()
  deps <- bslib::bs_theme_dependencies(theme)
  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")
  expect_match(css, "data-glass-preset")
  expect_match(css, "--glass-page-bg")
  expect_match(css, "--glass-body-color")
  # Both packs present
  expect_true(grepl("data-glass-preset=\"light\"", css, fixed = TRUE) ||
    grepl("data-glass-preset=.light", css))
  expect_true(grepl("data-glass-preset=\"dark\"", css, fixed = TRUE) ||
    grepl("data-glass-preset=.dark", css))
  # Runtime page background
  expect_match(css, "var\\(--glass-page-bg\\)")
  # Layout contract from 0.1.1
  expect_match(css, "glass-sidebar-reserve")
  expect_match(css, "position:absolute\\s*!important")
  expect_match(css, "width:var\\(--_sidebar-width")
  expect_match(css, "\\.bslib-sidebar-layout \\.bslib-sidebar-layout")
  expect_match(css, "position:relative\\s*!important")
  expect_match(css, "main-sidebar")
  expect_match(css, "small-box")
  # White checkmark on checked checkboxes (not body-color / black stroke)
  expect_match(css, "form-check-input")
  expect_match(css, "stroke='%23fff'|stroke=\"%23fff\"|stroke='%23FFF'")
})

test_that("active-on-primary ink is light, not color-contrast black", {
  skip_if_not_installed("bslib")
  # Bootstrap color-contrast(#007AFF) is black at 4.5:1; glass_theme must fix.
  th <- glass_theme(primary = "#007AFF")
  vars <- bslib::bs_get_variables(
    th,
    c(
      "min-contrast-ratio",
      "component-active-color",
      "form-check-input-checked-color",
      "form-check-input-indeterminate-color",
      "form-switch-checked-color",
      "pagination-active-color",
      "component-active-bg"
    )
  )
  # Core fix: lower Bootstrap's contrast gate so color-contrast() picks white
  # on brand blues (white on #007AFF is ~4.02:1).
  expect_equal(as.numeric(vars[["min-contrast-ratio"]]), 3)
  expect_equal(toupper(vars[["component-active-color"]]), "#FFFFFF")
  expect_equal(toupper(vars[["form-check-input-checked-color"]]), "#FFFFFF")
  expect_equal(toupper(vars[["form-check-input-indeterminate-color"]]), "#FFFFFF")
  expect_equal(toupper(vars[["form-switch-checked-color"]]), "#FFFFFF")
  expect_equal(toupper(vars[["pagination-active-color"]]), "#FFFFFF")
  expect_equal(toupper(vars[["component-active-bg"]]), "#007AFF")

  deps <- bslib::bs_theme_dependencies(th)
  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")

  # Checked / indeterminate / switch should not keep black SVG marks on primary
  # (our CSS safety net + Sass vars). Allow black only outside checked states.
  checked_black <- grepl(
    "form-check-input:checked[^}]*stroke='%23000'|form-switch[^\"]*fill='%23000'",
    css
  )
  expect_false(checked_black)
  expect_match(css, "form-switch")
  expect_match(css, "indeterminate")

  # text-bg-primary must compile light ink (not color-contrast black)
  expect_true(grepl("text-bg-primary\\{[^}]*color:#fff", css))
  # Solid .bg-primary still carries glass-on ink vars; value boxes are
  # tinted glass with body ink (not white-on-solid).
  expect_match(css, "--glass-on-primary")
  expect_match(css, "bslib-value-box-color")
  expect_match(css, "bslib-value-box\\.bg-primary|bslib-value-box.bg-primary")
})

test_that("glass_on_color matches luminance threshold", {
  # Saturated blue → white ink; light gold → dark ink
  expect_equal(tolower(.glass_on_color("#007AFF")), "#ffffff")
  expect_equal(tolower(.glass_on_color("#AF52DE")), "#ffffff")
  expect_equal(tolower(.glass_on_color("#FFCC00")), "#1d1d1f")
})

test_that("head script sets glass-on-primary for solid fills", {
  skip_if_not_installed("bslib")
  th <- glass_theme(primary = "#007AFF")
  deps <- bslib::bs_theme_dependencies(th)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  head <- preset_deps[[1]]$head
  expect_match(head, "--glass-on-primary")
  expect_match(head, "#ffffff|#FFFFFF")
})

test_that("update_glass_theme sends shinyglass custom message", {
  skip_if_not_installed("shiny")
  msgs <- list()
  session <- list(
    sendCustomMessage = function(type, message) {
      msgs[[length(msgs) + 1L]] <<- list(type = type, message = message)
    }
  )
  update_glass_theme(session, preset = "dark", tint = FALSE, primary = "#AF52DE")
  # Dual-channel: structured shinyglass + legacy glassPreset when preset set
  expect_length(msgs, 2)
  types <- vapply(msgs, `[[`, character(1), "type")
  expect_true("shinyglass" %in% types)
  expect_true("glassPreset" %in% types)
  sg <- msgs[[which(types == "shinyglass")[[1]]]]
  expect_equal(sg$message$preset, "dark")
  expect_equal(sg$message$tint, FALSE)
  expect_equal(sg$message$primary, "#AF52DE")
  expect_equal(msgs[[which(types == "glassPreset")[[1]]]]$message, "dark")
})

test_that("update_glass_theme validates preset", {
  session <- list(sendCustomMessage = function(...) NULL)
  expect_error(update_glass_theme(session, preset = "neon"), "arg|preset")
  expect_error(update_glass_theme(NULL, preset = "dark"))
})

test_that("glass_theme_toggle returns labeled buttons", {
  skip_if_not_installed("shiny")
  ui <- glass_theme_toggle(inputId = "gt", selected = "dark")
  expect_s3_class(ui, "shiny.tag")
  html <- as.character(ui)
  expect_match(html, "gt_light")
  expect_match(html, "gt_dark")
  expect_match(html, "gt_auto")
  expect_match(html, "glass-theme-toggle")
  expect_match(html, "data-glass-preset-mode")
  expect_match(html, "aria-pressed=\"true\"")
  expect_match(html, "btn-primary")
  expect_match(html, "btn-secondary")
})

test_that("head script includes primary CSS variables", {
  skip_if_not_installed("bslib")
  th <- glass_theme(primary = "#AF52DE")
  deps <- bslib::bs_theme_dependencies(th)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  head <- preset_deps[[1]]$head
  expect_match(head, "#AF52DE")
  expect_match(head, "--bs-primary")
  expect_match(head, "--bs-primary-rgb")
})

test_that("compiled CSS prefers reduced-motion and runtime primary hooks", {
  skip_if_not_installed("bslib")
  theme <- glass_theme()
  deps <- bslib::bs_theme_dependencies(theme)
  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")
  expect_match(css, "prefers-reduced-motion")
  expect_match(css, "--glass-accent")
  expect_match(css, "--bs-primary")
  expect_match(css, "shiny-plot-output")
  # Shiny checkboxInput markup (not only Bootstrap form-check-input)
  expect_match(css, "shiny-input-checkbox")
  expect_match(css, "glass-theme-toggle")
  expect_match(css, "--glass-body-color")
})

test_that("dark pack does not re-lock default button labels to $primary", {
  scss <- paste(readLines(system.file("scss", "glass.scss", package = "shinyglass"), warn = FALSE), collapse = "\n")
  expect_false(grepl(
    "color-mix\\(in srgb, \\$primary 88%, #ffffff 12%\\)",
    scss
  ))
})

test_that("glass_resolved_preset prefers client resolved value", {
  expect_equal(glass_resolved_preset(NULL), "light")
  expect_equal(glass_resolved_preset(list(), default = "dark"), "dark")
  expect_equal(
    glass_resolved_preset(list(glass_resolved_preset = "dark", preset = "auto")),
    "dark"
  )
  expect_equal(
    glass_resolved_preset(list(preset = "dark")),
    "dark"
  )
  expect_equal(
    glass_resolved_preset(list(preset = "auto")),
    "light"
  )
})

test_that("compiled CSS styles DT/Bootstrap page-link surfaces (not li-only)", {
  skip_if_not_installed("bslib")
  # Regression guard for black DT chips: DT Bootstrap 5 uses
  # <li.paginate_button.page-item><a.page-link>. Glass must paint .page-link.
  theme <- glass_theme()
  deps <- bslib::bs_theme_dependencies(theme)
  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")
  expect_match(css, "dataTables_paginate")
  expect_match(css, "page-link")
  expect_match(css, "page-item\\.active|page-item.active")
  expect_match(css, "page-item\\.disabled|paginate_button\\.disabled")
  # Visible chip uses glass surface tokens
  expect_true(grepl(
    "dataTables_paginate[^}]*page-link|page-link[^}]*--glass-bg|page-link[^;]*var\\(--glass-bg\\)",
    css
  ))
})

test_that("compiled CSS covers plotly, gt, waiter, and AdminLTE-plus chrome", {
  skip_if_not_installed("bslib")
  theme <- glass_theme()
  deps <- bslib::bs_theme_dependencies(theme)
  css_chunks <- character()
  for (d in deps) {
    src <- d$src$file %||% d$src
    sheets <- d$stylesheet
    if (is.null(sheets)) next
    for (f in if (is.list(sheets)) unlist(sheets) else sheets) {
      path <- file.path(src, f)
      if (file.exists(path)) {
        css_chunks <- c(css_chunks, paste(readLines(path, warn = FALSE), collapse = "\n"))
      }
    }
  }
  css <- paste(css_chunks, collapse = "\n")
  expect_match(css, "js-plotly-plot")
  expect_match(css, "modebar")
  expect_match(css, "gt_table")
  expect_match(css, "gt_col_heading")
  expect_match(css, "waiter-overlay")
  expect_match(css, "swal2-popup")
  expect_match(css, "user-panel")
  expect_match(css, "handsontable")
  expect_match(css, "datepicker")
  expect_match(css, "shiny-notification")
  expect_match(css, "accordion-button")
  expect_match(css, "btn-close")
})

test_that("audit-glass-contrast harness exists in the source tree", {
  # inst/scripts is intentionally .Rbuildignore'd for CRAN (devtools-only QA).
  # Assert the harness when developing from a full git checkout.
  root <- testthat::test_path("../..")
  audit_r <- file.path(root, "inst", "scripts", "audit-glass-contrast.R")
  audit_js <- file.path(root, "inst", "scripts", "audit-glass-contrast.js")
  qa_md <- file.path(root, "inst", "scripts", "VISUAL-QA.md")
  skip_if_not(
    file.exists(audit_r) && file.exists(audit_js),
    "inst/scripts not present (CRAN tarball / installed package)"
  )
  expect_true(file.exists(qa_md))
  js <- paste(readLines(audit_js, warn = FALSE), collapse = "\n")
  expect_match(js, "runGlassAudit")
  expect_match(js, "solid-black-chip|dt-pagination-structure")
  r_src <- paste(readLines(audit_r, warn = FALSE), collapse = "\n")
  expect_match(r_src, "chrome-kitchen-sink")
  expect_match(r_src, "open-datepicker")
})


test_that("glass_theme intensity is marked in head", {
  skip_if_not_installed("bslib")
  th <- glass_theme(intensity = 0.2)
  deps <- bslib::bs_theme_dependencies(th)
  preset_deps <- deps[vapply(deps, function(d) d$name, character(1)) == "shinyglass-preset"]
  expect_length(preset_deps, 1)
  expect_match(preset_deps[[1]]$head, "glassIntensity=0\\.2000")
  expect_match(preset_deps[[1]]$head, "--glass-intensity")
})

test_that("glass_intensity_slider returns a tag with range input", {
  skip_if_not_installed("htmltools")
  ui <- glass_intensity_slider("gi", value = 0.7, preview = FALSE)
  expect_s3_class(ui, "shiny.tag")
  html <- as.character(ui)
  expect_match(html, "glass-intensity-slider")
  expect_match(html, 'id="gi"')
  expect_match(html, 'value="0.7"')
  expect_match(html, "Ultra Clear")
  expect_match(html, "Tinted")
})

test_that("intensity must be in 0..1", {
  expect_error(glass_theme(intensity = 1.5), "intensity")
  expect_error(glass_theme(intensity = -0.1), "intensity")
})

test_that("update_glass_theme accepts intensity", {
  skip_if_not_installed("shiny")
  # session mock: capture sendCustomMessage
  msgs <- list()
  session <- list(
    sendCustomMessage = function(type, message) {
      msgs[[length(msgs) + 1L]] <<- list(type = type, message = message)
    }
  )
  class(session) <- "ShinySession"
  update_glass_theme(session, intensity = 0.8)
  expect_equal(length(msgs), 1L)
  expect_equal(msgs[[1]]$type, "shinyglass")
  expect_equal(msgs[[1]]$message$intensity, 0.8)
})

test_that("update_glass_theme dual-channels preset", {
  skip_if_not_installed("shiny")
  msgs <- list()
  session <- list(
    sendCustomMessage = function(type, message) {
      msgs[[length(msgs) + 1L]] <<- list(type = type, message = message)
    }
  )
  class(session) <- "ShinySession"
  update_glass_theme(session, preset = "dark")
  types <- vapply(msgs, `[[`, character(1), "type")
  expect_true("shinyglass" %in% types)
  expect_true("glassPreset" %in% types)
  sg <- msgs[[which(types == "shinyglass")[1]]]
  expect_equal(sg$message$preset, "dark")
  leg <- msgs[[which(types == "glassPreset")[1]]]
  expect_equal(leg$message, "dark")
})

test_that("glass_preset_input marks select for client binding", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("htmltools")
  ui <- glass_preset_input("my_preset", selected = "dark")
  html <- paste(as.character(ui), collapse = "\n")
  expect_match(html, "data-glass-preset-input")
  expect_match(html, "glass-preset-input")
  expect_match(html, 'id="my_preset"')
  expect_match(html, "dark")
})

# Showcase every standard Shiny input with shinyglass.
#
# Run:
#   shiny::runApp(system.file("examples", "inputs-gallery.R", package = "shinyglass"))
#
# Dark preset:
#   Sys.setenv(SHINYGLASS_PRESET = "dark")
#   shiny::runApp(system.file("examples", "inputs-gallery.R", package = "shinyglass"))

library(shiny)
library(bslib)
library(shinyglass)

`%||%` <- function(x, y) {
  if (is.null(x) || length(x) == 0L || (is.character(x) && !nzchar(x[[1]]))) y else x
}

glass_preset <- match.arg(
  Sys.getenv("SHINYGLASS_PRESET", "auto"),
  c("light", "dark", "auto")
)

ui <- page_sidebar(
  title = "Shiny Inputs Gallery",
  theme = glass_theme(preset = glass_preset),
  fillable = TRUE,
  sidebar = sidebar(
    title = "About",
    width = 280,
    open = "desktop",
    p(
      class = "text-muted",
      "Every built-in Shiny input and button, styled with ",
      code("glass_theme()"), ". Change controls — values update live below."
    ),
    glass_preset_input(
      "preset",
      label = "Theme preset",
      selected = glass_preset,
      width = "100%"
    ),
    tags$hr(),
    tags$small(
      class = "text-muted",
      "Sidebar starts closed on phones; use the toggle to open filters."
    )
  ),
  # Stack cards on phones (min ~16rem) instead of fixed 50/50 columns
  layout_column_wrap(
    width = "16rem",
    heights_equal = "row",
    gap = "0.75rem",
    card(
      card_header("Text & numbers"),
      textInput("text", "textInput", "Hello glass", width = "100%"),
      passwordInput("password", "passwordInput", "secret", width = "100%"),
      textAreaInput("textarea", "textAreaInput", "Multi-line\ntext", rows = 3, width = "100%"),
      numericInput("numeric", "numericInput", 42, min = 0, max = 100, width = "100%")
    ),
    card(
      card_header("Slider"),
      sliderInput("slider", "sliderInput", 0, 100, 50, step = 5, width = "100%"),
      sliderInput(
        "slider_range",
        "sliderInput (range)",
        min = 0,
        max = 100,
        value = c(25, 75),
        width = "100%"
      )
    ),
    card(
      card_header("Select menus"),
      selectInput(
        "select",
        "selectInput",
        c("Apple" = "apple", "Banana" = "banana", "Cherry" = "cherry"),
        selected = "banana",
        width = "100%"
      ),
      selectizeInput(
        "selectize",
        "selectizeInput",
        c("Red" = "red", "Green" = "green", "Blue" = "blue"),
        selected = "blue",
        multiple = TRUE,
        width = "100%"
      ),
      selectizeInput(
        "selectize_single",
        "selectizeInput (single)",
        state.name,
        selected = "California",
        width = "100%"
      )
    ),
    card(
      card_header("Checkboxes & radios"),
      checkboxInput("checkbox", "checkboxInput", TRUE),
      checkboxGroupInput(
        "checkbox_group",
        "checkboxGroupInput",
        c("Email" = "email", "SMS" = "sms", "Push" = "push"),
        selected = c("email", "push"),
        inline = FALSE
      ),
      radioButtons(
        "radio",
        "radioButtons",
        c("Small" = "sm", "Medium" = "md", "Large" = "lg"),
        selected = "md",
        inline = FALSE
      )
    ),
    card(
      card_header("Variable selectors"),
      varSelectInput("var_select", "varSelectInput", mtcars, selected = "mpg", width = "100%"),
      varSelectizeInput(
        "var_selectize",
        "varSelectizeInput",
        iris,
        selected = "Species",
        multiple = TRUE,
        width = "100%"
      )
    ),
    card(
      card_header("Dates"),
      dateInput("date", "dateInput", Sys.Date(), width = "100%"),
      dateRangeInput(
        "date_range",
        "dateRangeInput",
        start = Sys.Date() - 7,
        end = Sys.Date(),
        width = "100%"
      )
    ),
    card(
      card_header("File upload"),
      fileInput("file", "fileInput", accept = c(".txt", ".csv", ".pdf"), width = "100%")
    ),
    card(
      card_header("Buttons & links"),
      div(
        class = "d-flex flex-wrap gap-2 align-items-center",
        actionButton("action", "actionButton", class = "btn-primary"),
        actionLink("action_link", "actionLink")
      ),
      div(
        class = "d-flex flex-wrap gap-2 align-items-center mt-2",
        # Client-side BS5 modal open (works even if Shiny custom messages lag)
        tags$button(
          type = "button",
          class = "btn btn-outline-primary",
          id = "show_modal",
          `data-bs-toggle` = "modal",
          `data-bs-target` = "#glass_demo_modal",
          "Show modal"
        ),
        downloadButton("download", "downloadButton"),
        downloadLink("download_link", "downloadLink")
      ),
      tags$form(
        class = "mt-2",
        textInput("submit_text", NULL, "Form field", width = "100%"),
        submitButton("submitButton")
      )
    ),
    card(
      card_header("Live values"),
      class = "bslib-card",
      # renderPrint + verbatimTextOutput is the reliable pair for str()-style dumps
      verbatimTextOutput("values", placeholder = TRUE)
    )
  ),
  # Static Bootstrap 5 modal (client open) so it works even if custom messages
  # are flaky; glass_theme styles .modal-content + .modal-backdrop blur.
  tags$div(
    class = "modal fade",
    id = "glass_demo_modal",
    tabindex = "-1",
    `aria-labelledby` = "glass_demo_modal_title",
    `aria-hidden` = "true",
    tags$div(
      class = "modal-dialog modal-dialog-centered",
      tags$div(
        class = "modal-content",
        tags$div(
          class = "modal-header",
          tags$h5(
            class = "modal-title",
            id = "glass_demo_modal_title",
            "Glass modal"
          ),
          tags$button(
            type = "button",
            class = "btn-close",
            `data-bs-dismiss` = "modal",
            `aria-label` = "Close"
          )
        ),
        tags$div(
          class = "modal-body",
          p(
            "This dialog is a Bootstrap modal under ",
            code("glass_theme()"),
            ". The page behind frosts with backdrop blur (same idea as the file-picker scrim)."
          ),
          selectInput(
            "modal_choice",
            "Modal selectInput",
            c("Ultra Clear", "Balanced", "Tinted"),
            selected = "Balanced",
            width = "100%"
          ),
          textInput(
            "modal_text",
            "Modal textInput",
            "Hello from the modal",
            width = "100%"
          )
        ),
        tags$div(
          class = "modal-footer",
          tags$button(
            type = "button",
            class = "btn btn-secondary",
            `data-bs-dismiss` = "modal",
            "Close"
          ),
          # data-bs-dismiss closes on the client; server still gets the click
          actionButton(
            "modal_ok",
            "OK",
            class = "btn-primary",
            `data-bs-dismiss` = "modal"
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {
  # Client applies preset live; this keeps the server session aligned
  observe_glass_preset_input(input, session, "preset")

  observeEvent(input$action, {
    showNotification("actionButton clicked", type = "message", duration = 3)
  })

  observeEvent(input$action_link, {
    showNotification("actionLink clicked", type = "message", duration = 3)
  })

  # Client data-bs-toggle / data-bs-dismiss open and close the modal.
  observeEvent(input$modal_ok, {
    showNotification(
      paste0(
        "Modal OK - ",
        input$modal_choice %||% "?",
        " / ",
        input$modal_text %||% ""
      ),
      type = "message",
      duration = 3
    )
  })

  output$download <- downloadHandler(
    filename = function() {
      paste0("shinyglass-inputs-", format(Sys.Date(), "%Y%m%d"), ".csv")
    },
    content = function(file) {
      write.csv(mtcars, file, row.names = FALSE)
    }
  )

  output$download_link <- downloadHandler(
    filename = function() {
      paste0("shinyglass-inputs-", format(Sys.Date(), "%Y%m%d"), ".txt")
    },
    content = function(file) {
      writeLines(c("shinyglass inputs gallery", capture.output(str(reactiveValuesToList(input)))), file)
    }
  )

  output$values <- renderPrint({
    file_label <- if (is.null(input$file)) {
      NULL
    } else {
      list(
        name = input$file$name,
        size = input$file$size,
        type = input$file$type
      )
    }

    # Avoid str() inside renderText (returns NULL). renderPrint captures it.
    list(
      textInput = input$text,
      passwordInput = if (isTruthy(input$password) && nzchar(input$password)) "<redacted>" else "",
      textAreaInput = input$textarea,
      numericInput = input$numeric,
      sliderInput = input$slider,
      sliderInput_range = input$slider_range,
      selectInput = input$select,
      selectizeInput = input$selectize,
      selectizeInput_single = input$selectize_single,
      checkboxInput = input$checkbox,
      checkboxGroupInput = input$checkbox_group,
      radioButtons = input$radio,
      varSelectInput = as.character(input$var_select),
      varSelectizeInput = as.character(input$var_selectize),
      dateInput = as.character(input$date),
      dateRangeInput = as.character(input$date_range),
      fileInput = file_label,
      submitButton = input$submit_text,
      theme_preset = input$preset,
      modal_choice = input$modal_choice,
      modal_text = input$modal_text
    )
  })
}

shinyApp(ui, server)

---
title: "Ecosystem compatibility"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Ecosystem compatibility}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

shinyglass is a **theme layer**, not a universal skin. It targets Bootstrap 5 /
bslib surfaces first, then overlays denser dashboard stacks where practical.

# Support matrix (as of 0.2)

| Stack | Status | Notes |
|-------|--------|-------|
| `fluidPage` / `navbarPage` | Supported | Drop-in `theme = glass_theme()` |
| bslib `page_navbar` / `nav_menu` | Supported | See `chrome-kitchen-sink.R` |
| bslib `page_sidebar` / `layout_sidebar` | Supported | Floating glass sidebar; nested layouts stay in-flow (0.1.1+) |
| bslib cards, value boxes, navsets, accordion | Supported | Tooltip / popover / toast use the glass menu surface |
| DT | Supported | Visible `.page-link` chips (not li-only) |
| reactable | Supported | Avoids double-framing the html-output |
| **gt** | Supported | Table / heading / stripe / summary chrome |
| leaflet | Supported | Map container transparency |
| **plotly** | Good | Glass frame + modebar; set `paper_bgcolor` transparent in R for best dark |
| shinyWidgets (common inputs) | Good | Slider tooltips / noUiSlider chips |
| shinydashboard / **bs4Dash** / **shinydashboardPlus** | Overlay | Glass header/sidebar/boxes/controlbar/dropdowns; layout stays AdminLTE |
| teal | Good | Use `options(teal.bs_theme = glass_theme())` |
| **waiter** / shinybusy / loaders | Good | Blurred overlays, not solid white scrims |
| **shinyalert** / SweetAlert2 | Good | Popup uses glass menu surface |
| rhandsontable | Partial | Frame + cell ink; core grid still third-party |
| echarts4r / highcharter / apex | Partial | Host wrapper glass; chart internals set in R |
| Bootstrap 3-only apps | Limited | Theme is BS5 via bslib |

# Layout contract

- Open bslib sidebars **reserve** space in main so content is not buried under
  the floating chrome.
- Nested `layout_sidebar()` does **not** double-float.
- AdminLTE left sidebars keep fixed AdminLTE geometry; glass is visual overlay
  only.

# Examples

```r
# overlays: datepicker, notifications, showModal, accordion, navbar menu
shiny::runApp(system.file("examples", "chrome-kitchen-sink.R", package = "shinyglass"))

# plotly + gt (+ waiter if installed)
shiny::runApp(system.file("examples", "plotly-gt-demo.R", package = "shinyglass"))

# denser AdminLTE (requires shinydashboardPlus)
shiny::runApp(system.file("examples", "shinydashboardPlus-glass-demo.R", package = "shinyglass"))
```

Dual-theme contrast audit (optional ecosystem apps skip if packages missing):

```r
Rscript inst/scripts/audit-glass-contrast.R --apps=demo,dashboard,inputs,plotly_gt,chrome
```

# Tips

- Prefer bslib page functions when you can — glass was designed around them.
- For plotly dark mode, pass transparent `paper_bgcolor` / `plot_bgcolor` in
  `layout()` (see `plotly-gt-demo.R`); CSS handles the modebar.
- For dark mode in dense AdminLTE apps, re-check value/info box contrast after
  switching preset at runtime.
- `preset = "auto"` is ideal for demos; pin `"light"` or `"dark"` for
  screenshots and tests.

---
title: "Theming and light/dark"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Theming and light/dark}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

# Basics

```{r}
library(shiny)
library(shinyglass)

ui <- fluidPage(
  theme = glass_theme(), # or preset = "dark" / "auto"
  titlePanel("Liquid Glass"),
  plotOutput("p")
)
```

`glass_theme()` returns a **bslib** theme. Pass it as `theme =` to
`fluidPage()`, `navbarPage()`, `bslib::page_sidebar()`, and other page
helpers that accept a bslib theme.

# Presets

| `preset` | Behavior |
|----------|----------|
| `"light"` | Light glass pack (default) |
| `"dark"` | Dark glass pack |
| `"auto"` | Follows `prefers-color-scheme`; updates when the OS theme changes |

Light and dark surface tokens ship as dual CSS custom-property packs on
`[data-glass-preset]`. Switching preset updates
`document.documentElement.dataset.glassPreset` — no Sass recompile and no
full page reload.

# Runtime updates from the server

```{r}
ui <- fluidPage(
  theme = glass_theme(preset = "auto"),
  glass_theme_toggle(selected = "auto"),
  selectInput("accent", "Accent", c(
    Blue = "#007AFF", Purple = "#AF52DE", Orange = "#FF9500"
  ))
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)
  observeEvent(input$accent, {
    update_glass_theme(session, primary = input$accent)
  }, ignoreInit = TRUE)
}
```

`update_glass_theme()` accepts:

| Argument | Effect |
|----------|--------|
| `preset` | `"light"` / `"dark"` / `"auto"` |
| `tint` | Content-aware ambient color from plots/images |
| `primary` | Live accent via CSS variables (`--bs-primary`, …) |

`primary` updates Bootstrap accent CSS variables so buttons, checked
controls, and other accent surfaces follow without a reload. A few
Sass-baked one-offs may still need a full reload to pick up a new color.

# Theme toggle helper

```{r}
ui <- fluidPage(
  theme = glass_theme(preset = "auto"),
  glass_theme_toggle(),           # Light / Dark / Auto buttons
  # ...
)

server <- function(input, output, session) {
  observe_glass_theme_toggle(input, session)
}
```

Buttons call `window.shinyglass.setPreset()` immediately and also fire
Shiny inputs (`glass_toggle_light`, …) so the server can stay in sync.

# Behavior knobs

```{r}
glass_theme(
  preset = "auto",
  primary = "#AF52DE",
  blur = 36,
  saturation = 200,
  radius = "1.5rem",
  material = "regular", # or "clear" over media-rich content
  intensity = 0.45,     # 0 Ultra Clear → 1 Tinted (iOS 27)
  tint = TRUE,      # sample plot/image colors into glass surfaces
  specular = TRUE,  # pointer specular highlight
  nav_morph = TRUE  # compact navbar while scrolling down
)
```

`material = "regular"` is the adaptive Tahoe-style fill (default). Use
`"clear"` when chrome sits over rich media and labels stay bold.

Use [glass_intensity_slider()] for a live Ultra Clear → Tinted control (same
idea as iOS 27 Settings → Appearance → Liquid Glass):

```{r}
glass_intensity_slider("glass_intensity", value = 0.45)
```

All three JS behaviors default to `TRUE` (same as 0.1.x). They respect
`prefers-reduced-motion: reduce` where relevant. Advanced escape hatch:
`window.__shinyglassDisableTint = true` still disables tint.

# CSS variables (migration from 0.1.x)

**Breaking in 0.2.0:** light/dark surfaces are dual CSS packs, not a
Bootswatch `darkly` rebuild. Prefer CSS custom properties over Sass
`$glass-*` tokens when customizing:

```css
:root {
  /* shared knobs also set by glass_theme(blur=..., radius=...) */
  --glass-blur: 36px;
  --glass-radius: 1.25rem;
  --bs-primary: #007AFF;       /* also updated by update_glass_theme(primary=) */
  --glass-primary: #007AFF;
}

:root[data-glass-preset="dark"] {
  /* override a dark-pack value if needed */
  --glass-bg: rgba(255, 255, 255, 0.1);
}
```

Apps that overrode Sass-only `$glass-*` tokens should target `--glass-*`
or `[data-glass-preset="dark"]` instead.

# Client helpers

```js
window.shinyglass.setPreset("dark"); // or "light" / "auto"
window.shinyglass.getPreset();       // resolved "light" | "dark"
window.shinyglass.getMode();         // requested mode incl. "auto"
window.shinyglass.setTint(false);
window.shinyglass.setPrimary("#AF52DE");
window.shinyglass.getPrimary();
```

# Teal

```{r}
options(teal.bs_theme = glass_theme(preset = "auto"))
# then teal::init(...) as usual
```

# Reduced motion

When the user prefers reduced motion (`prefers-reduced-motion: reduce`):

- Navbar morph transforms are disabled (JS + CSS)
- Pointer specular tracking is skipped
- Content tint sampling is skipped
- Decorative transitions on cards/inputs/nav are removed

This is automatic — no app code required.

# Active-on-accent contrast

Bootstrap’s `color-contrast()` often picks **black** ink for system blue
`#007AFF`. `glass_theme()` forces light ink on primary fills so checked
checkboxes, radios, switches, and active pagination stay readable.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{update_glass_theme}
\alias{update_glass_theme}
\title{Update glass theme options in a running app}
\usage{
update_glass_theme(
  session,
  preset = NULL,
  tint = NULL,
  primary = NULL,
  intensity = NULL
)
}
\arguments{
\item{session}{A Shiny session object (usually the \code{session} argument of
the server function).}

\item{preset}{Optional. \code{"light"}, \code{"dark"}, or \code{"auto"}.}

\item{tint}{Optional logical. Enable or disable content-aware ambient tint.}

\item{primary}{Optional accent color (hex like \code{"#AF52DE"} or \code{rgb()}).}

\item{intensity}{Optional numeric in \eqn{[0, 1]}: Ultra Clear (\code{0}) to
Tinted (\code{1}).}
}
\value{
\code{session}, invisibly.
}
\description{
Send a message to the browser to change the Liquid Glass preset, accent
color, intensity, or content-tint behavior without reloading the page.
Requires a page that used \code{\link[=glass_theme]{glass_theme()}} (so \code{shiny-glass.js} is loaded).
}
\details{
\code{primary} updates CSS variables (\code{--bs-primary}, \code{--bs-primary-rgb},
\code{--glass-primary}) so buttons, checks, and other accent surfaces follow the
new color. Sass-baked one-off colors may not all switch until a full reload.
\code{intensity} updates fill/blur along the Ultra Clear to Tinted spectrum
(see \code{\link[=glass_intensity_slider]{glass_intensity_slider()}}).
}
\examples{
if (interactive()) {
  library(shiny)
  library(shinyglass)

  ui <- fluidPage(
    theme = glass_theme(),
    glass_theme_toggle(),
    selectInput("accent", "Accent", c("#007AFF", "#AF52DE", "#FF9500"))
  )

  server <- function(input, output, session) {
    observe_glass_theme_toggle(input, session)
    observeEvent(input$accent, {
      update_glass_theme(session, primary = input$accent)
    }, ignoreInit = TRUE)
  }

  shinyApp(ui, server)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{glass_theme}
\alias{glass_theme}
\title{Liquid Glass theme for 'shiny'}
\usage{
glass_theme(
  preset = c("light", "dark", "auto"),
  primary = "#007AFF",
  blur = 36,
  saturation = 200,
  radius = "1.5rem",
  material = c("regular", "clear"),
  intensity = 0.45,
  tint = TRUE,
  specular = TRUE,
  nav_morph = TRUE,
  ...
)
}
\arguments{
\item{preset}{\code{"light"}, \code{"dark"}, or \code{"auto"}. \code{"auto"} follows
\code{prefers-color-scheme} and updates when the OS theme changes.}

\item{primary}{Accent color for buttons, links, and focus rings.
Defaults to system blue (\verb{#007AFF}).}

\item{blur}{Backdrop blur radius in pixels. Default \code{36} matches the
iOS 27 diffusion-first material.}

\item{saturation}{Backdrop saturation percentage.}

\item{radius}{Default border radius for glass surfaces (CSS length).
Prefer larger concentric radii (default \verb{1.5rem}).}

\item{material}{\code{"regular"} (adaptive, most UI) or \code{"clear"} (more
transparent; best over media-rich content with bold labels).}

\item{intensity}{Liquid Glass intensity from \code{0} (Ultra Clear) to \code{1}
(Tinted), matching iOS 27 Appearance -> Liquid Glass. Default \code{0.45}.
Use \code{\link[=glass_intensity_slider]{glass_intensity_slider()}} for a live control.}

\item{tint}{Content-aware ambient tint from plots/images (JS).}

\item{specular}{Pointer-driven specular highlight on glass surfaces (JS).}

\item{nav_morph}{Compact navbar on scroll down; expand on scroll up (JS).}

\item{...}{Additional arguments forwarded to \code{\link[bslib:bs_theme]{bslib::bs_theme()}}.}
}
\value{
A \code{\link[bslib:bs_theme]{bslib::bs_theme()}} object suitable for 'shiny' page functions.
}
\description{
Create a \code{\link[bslib:bs_theme]{bslib::bs_theme()}} styled with a Liquid Glass look: translucent
surfaces, backdrop blur, soft depth, and system typography. Pass the result
to \verb{theme =} on \code{fluidPage()}, \code{navbarPage()}, \code{bslib::page_sidebar()}, or
any other page function that accepts a bslib theme.
}
\details{
Light and dark surface tokens are compiled into dual CSS custom-property
packs. Switching \code{preset} at runtime (via \code{\link[=update_glass_theme]{update_glass_theme()}} or
\code{preset = "auto"}) updates \code{document.documentElement.dataset.glassPreset}
without recompiling Sass or reloading the page. Accent color can also be
updated live with \code{\link[=update_glass_theme]{update_glass_theme()}} \verb{primary=} (CSS variables).
}
\examples{
theme <- glass_theme()
dark <- glass_theme(preset = "dark", primary = "#BF5AF2")
auto <- glass_theme(preset = "auto", tint = FALSE)
clear <- glass_theme(material = "clear")

if (interactive()) {
  library(shiny)

  ui <- fluidPage(
    theme = glass_theme(preset = "auto"),
    titlePanel("Liquid Glass"),
    glass_theme_toggle(),
    selectInput("color", "Color", c("Blue", "Purple", "Orange")),
    plotOutput("plot")
  )

  server <- function(input, output, session) {
    # Client onclick already switches; keep session in sync:
    observe_glass_theme_toggle(input, session)
  }

  shinyApp(ui, server)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinyglass-package.R
\docType{package}
\name{shinyglass-package}
\alias{shinyglass}
\alias{shinyglass-package}
\title{Liquid Glass Design Themes for 'shiny' Applications}
\description{
Drop-in Liquid Glass themes for \href{https://shiny.posit.co/}{shiny}. Call
\code{\link[=glass_theme]{glass_theme()}} and pass the result to \verb{theme =} on \code{fluidPage()},
\code{navbarPage()}, or any \href{https://rstudio.github.io/bslib/}{bslib}-aware page
function to get translucent surfaces, backdrop blur, and system typography.
}
\section{Getting started}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(shiny)
library(shinyglass)

ui <- fluidPage(
  theme = glass_theme(preset = "auto", intensity = 0.45),
  titlePanel("Liquid Glass"),
  glass_intensity_slider("glass_intensity"),
  glass_theme_toggle(selected = "auto"),
  sliderInput("n", "Bars", 5, 30, 15),
  plotOutput("plot")
)

server <- function(input, output, session) \{
  observe_glass_theme_toggle(input, session)
  observe_glass_intensity(input, session, "glass_intensity")
  output$plot <- renderPlot(
    barplot(seq_len(input$n), col = "#007AFF", border = NA)
  )
\}
}\if{html}{\out{</div>}}

Light and dark presets are available via \code{glass_theme(preset = "dark")},
or \code{preset = "auto"} to follow the OS. Switch at runtime with
\code{\link[=update_glass_theme]{update_glass_theme()}} or \code{\link[=glass_theme_toggle]{glass_theme_toggle()}}. Material density is
controlled with \code{\link[=glass_intensity_slider]{glass_intensity_slider()}} (Ultra Clear to Tinted) or
\code{glass_theme(intensity = )}. Accent color, blur, saturation, corner radius,
and JS behaviors (\code{tint}, \code{specular}, \code{nav_morph}) are configurable.

For \href{https://insightsengineering.github.io/teal/}{teal} apps, set
\code{options(teal.bs_theme = glass_theme())} before calling \code{teal::init()}.
}

\seealso{
\code{\link[=glass_theme]{glass_theme()}}, \code{\link[=glass_intensity_slider]{glass_intensity_slider()}}, \code{\link[=update_glass_theme]{update_glass_theme()}},
\code{\link[=glass_theme_toggle]{glass_theme_toggle()}}
}
\author{
\strong{Maintainer}: Eric Anderson \email{eric.ray.anderson@gmail.com} [copyright holder]

Authors:
\itemize{
  \item Eric Anderson \email{eric.ray.anderson@gmail.com} [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{glass_resolved_preset}
\alias{glass_resolved_preset}
\title{Resolved light or dark appearance}
\usage{
glass_resolved_preset(input, default = c("light", "dark"))
}
\arguments{
\item{input}{The server \code{input} object.}

\item{default}{Fallback if the client has not reported yet (\code{"light"} or
\code{"dark"}).}
}
\value{
\code{"light"} or \code{"dark"}.
}
\description{
When the user picks \strong{Auto}, \code{input$preset} stays \code{"auto"} so checks like
\code{identical(input$preset, "dark")} stay FALSE and plots keep light ink on a
dark page. The client publishes the \emph{resolved} appearance as
\code{input$glass_resolved_preset} (\code{"light"} or \code{"dark"}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{glass_preset_input}
\alias{glass_preset_input}
\title{Light / dark / auto preset select input}
\usage{
glass_preset_input(
  inputId = "glass_preset",
  label = "Theme preset",
  selected = c("auto", "light", "dark"),
  choices = c(Light = "light", Dark = "dark", `Auto (OS)` = "auto"),
  width = NULL
)
}
\arguments{
\item{inputId}{The \code{input} slot that will be used to access the value.}

\item{label}{Display label for the control (or \code{NULL} for none).}

\item{selected}{Initially selected mode (\code{"light"}, \code{"dark"}, or \code{"auto"}).}

\item{choices}{Named character vector of choices. Defaults to Light / Dark /
Auto (OS). Names are labels; values must be \code{light}, \code{dark}, and/or \code{auto}.}

\item{width}{The width of the input (e.g. \code{"100\%"}).}
}
\value{
A \code{\link[shiny:selectInput]{shiny::selectInput()}} tag (with glass client bindings).
}
\description{
Drop-in \code{\link[shiny:selectInput]{shiny::selectInput()}} for the glass theme preset. The client applies
the change \strong{immediately} via \code{window.shinyglass.setPreset()} (marked with
\code{data-glass-preset-input}), so Light/Dark/Auto works even when custom
messages are delayed or rewritten (e.g. on some shinyapps.io hosts). Pair
with \code{\link[=observe_glass_preset_input]{observe_glass_preset_input()}} so the server stays in sync.
}
\seealso{
\code{\link[=observe_glass_preset_input]{observe_glass_preset_input()}}, \code{\link[=glass_theme_toggle]{glass_theme_toggle()}},
\code{\link[=update_glass_theme]{update_glass_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-intensity.R
\name{observe_glass_intensity}
\alias{observe_glass_intensity}
\title{Keep session intensity in sync with \code{\link[=glass_intensity_slider]{glass_intensity_slider()}}}
\usage{
observe_glass_intensity(input, session, inputId = "glass_intensity")
}
\arguments{
\item{input}{The server \code{input} object.}

\item{session}{A Shiny session object.}

\item{inputId}{Input id of the intensity slider.}
}
\value{
An \code{\link[shiny:observeEvent]{shiny::observeEvent()}} observer (invisibly).
}
\description{
The slider already updates glass live on the client. This observer optionally
echoes the value through \code{\link[=update_glass_theme]{update_glass_theme()}} so other clients / server
state stay aligned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{observe_glass_preset_input}
\alias{observe_glass_preset_input}
\title{Observe \code{\link[=glass_preset_input]{glass_preset_input()}} on the server}
\usage{
observe_glass_preset_input(input, session, inputId = "glass_preset")
}
\arguments{
\item{input}{The server \code{input} object.}

\item{session}{The server \code{session} object.}

\item{inputId}{Same id passed to \code{\link[=glass_preset_input]{glass_preset_input()}}.}
}
\value{
An \code{\link[shiny:observeEvent]{shiny::observeEvent()}} observer (invisibly).
}
\description{
Wires a preset select to \code{\link[=update_glass_theme]{update_glass_theme()}} so session state stays
aligned with the client (the client already applied the preset live).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{observe_glass_theme_toggle}
\alias{observe_glass_theme_toggle}
\title{Observe \code{\link[=glass_theme_toggle]{glass_theme_toggle()}} buttons on the server}
\usage{
observe_glass_theme_toggle(input, session, inputId = "glass_toggle")
}
\arguments{
\item{input}{The server \code{input} object.}

\item{session}{The server \code{session} object.}

\item{inputId}{Same base id passed to \code{\link[=glass_theme_toggle]{glass_theme_toggle()}}.}
}
\value{
\code{NULL}, invisibly. Called for side effects (registers observers).
}
\description{
Wires the three toggle inputs to \code{\link[=update_glass_theme]{update_glass_theme()}} so session state
stays aligned with the client (needed on some hosts that rewrite 'shiny'
messaging).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-theme.R
\name{glass_theme_toggle}
\alias{glass_theme_toggle}
\title{Light / dark / auto theme toggle buttons}
\usage{
glass_theme_toggle(
  inputId = "glass_toggle",
  selected = c("auto", "light", "dark"),
  labels = c(light = "Light", dark = "Dark", auto = "Auto (OS)"),
  class = "glass-theme-toggle"
)
}
\arguments{
\item{inputId}{Base id. Buttons are \verb{\{inputId\}_light}, \verb{\{inputId\}_dark},
and \verb{\{inputId\}_auto}.}

\item{selected}{Initially highlighted mode (\code{"light"}, \code{"dark"}, or
\code{"auto"}). Cosmetic only; the page theme still comes from \code{\link[=glass_theme]{glass_theme()}}.}

\item{labels}{Named character vector for button labels. Names must be
\code{light}, \code{dark}, and/or \code{auto}.}

\item{class}{Extra CSS classes for the wrapper.}
}
\value{
An \code{\link[htmltools:tag]{htmltools::tag()}} button group.
}
\description{
Drop-in button group for switching the glass preset. Each button sets the
preset on the client immediately (\code{window.shinyglass.setPreset}) and also
has a 'shiny' input id so \code{\link[=observe_glass_theme_toggle]{observe_glass_theme_toggle()}} can keep the server
in sync (important on hosts that rewrite custom messages).
}
\seealso{
\code{\link[=observe_glass_theme_toggle]{observe_glass_theme_toggle()}}, \code{\link[=update_glass_theme]{update_glass_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glass-intensity.R
\name{glass_intensity_slider}
\alias{glass_intensity_slider}
\title{Liquid Glass intensity slider (iOS 27-style)}
\usage{
glass_intensity_slider(
  inputId = "glass_intensity",
  label = "Liquid Glass",
  value = NULL,
  min = 0,
  max = 1,
  step = 0.01,
  min_label = "Ultra Clear",
  max_label = "Tinted",
  preview = TRUE,
  width = NULL
)
}
\arguments{
\item{inputId}{The \code{input} slot that will be used to access the value.}

\item{label}{Display label for the control (or \code{NULL} for none).}

\item{value}{Initial intensity in \eqn{[0, 1]}. If \code{NULL}, uses the theme
default from \code{\link[=glass_theme]{glass_theme()}} / the client current intensity.}

\item{min, max, step}{Range for the underlying range input. Defaults cover
the full Ultra Clear -> Tinted spectrum.}

\item{min_label, max_label}{End-cap captions (iOS uses "Ultra Clear" /
"Tinted").}

\item{preview}{Show three mini glass chips as a live material sample.}

\item{width}{CSS width (passed to \code{\link[shiny:validateCssUnit]{shiny::validateCssUnit()}}).}
}
\value{
A Shiny UI tag hierarchy.
}
\description{
UI control matching iOS 27 \strong{Settings -> Appearance -> Liquid Glass}: a
continuous slider from \strong{Ultra Clear} (\code{0}) to \strong{Tinted} (\code{1}) that
live-updates the glass material without a page reload.
}
\details{
The client applies changes immediately via \code{window.shinyglass.setIntensity()}.
The value is also a normal Shiny input (\code{input[[inputId]]}) so the server can
react or persist it. Pair with \code{\link[=glass_theme]{glass_theme()}} \verb{intensity=} for the starting
value, and optionally \code{\link[=observe_glass_intensity]{observe_glass_intensity()}} to push server-driven
updates through \code{\link[=update_glass_theme]{update_glass_theme()}}.
}
\examples{
if (interactive()) {
  library(shiny)
  library(shinyglass)

  ui <- fluidPage(
    theme = glass_theme(intensity = 0.35),
    glass_theme_toggle(),
    glass_intensity_slider("glass_intensity"),
    plotOutput("p")
  )

  server <- function(input, output, session) {
    observe_glass_theme_toggle(input, session)
    # optional: mirror slider -> server message (client already updates live)
    observe_glass_intensity(input, session, "glass_intensity")
    output$p <- renderPlot(plot(rnorm(100), rnorm(100), pch = 16, col = "#007AFF"))
  }

  shinyApp(ui, server)
}

}
